﻿(function (PlayerFramework, undefined) {
    "use strict";

    // AdSdkPlugin Errors
    var invalidConstruction = "Invalid construction: AdSdkPlugin constructor must be called using the \"new\" operator.";

    // AdSdkPlugin Events
    var events = [
        "loadfailure",
        "adrefreshed",
        "erroroccurred"
    ];

    /// AdSdkPlugin Class
    /// <event name="adrefreshed">Occurs when the ad is refreshed/loaded by the AdSDK, does not mean MMPPF has loaded ad. 
    ///    No event params are passed in.</event>
    /// <event name="erroroccurred">Occurs when an ad failed to load by the AdSDK. Event.detail object contains the error code from AdSDK.</event>
    /// <param name="adUnitId">Gets or sets the ad unit identifier for this AdSDKPlugin instance. This value is provided to you when you register the app with pubCenter.</param>
    /// <param name="applicationId">Gets or sets the application ID of the app. This value is provided to you when you register the app with pubCenter.</param>
    /// <param name="adTags">Add ad tags to the ad control. The maximum is 10 tags per ad control. Ad tags are a key:value pair where key is maximum 16 characters and value is maximum 128 characters.</param>
    var AdSDKPlugin = WinJS.Class.derive(PlayerFramework.PluginBase, function (options) {
        if (!(this instanceof PlayerFramework.Plugins.AdSDKPlugin)) {
            throw invalidConstruction;
        }

        PlayerFramework.PluginBase.call(this, options);
    }, {
        // ========== Private Instance Methods
        _setOptions: function (options) {
            PlayerFramework.Utilities.setOptions(this, options, {
                isEnabled: true,
            });
        },

        _onActivate: function () {
            this._bindEvent("loading", this.mediaPlayer, this._onMediaPlayerLoading);

            return true;
        },

        _onDeactivate: function () {
            this._cancelActiveLoad();

            this._unbindEvent("loading", this.mediaPlayer, this._onMediaPlayerLoading);
        },

        _onLoad: function () {
        },

        _onUnload: function () {
        },

        _onErrorOccurred: function (evtArgs) {
            this.dispatchEvent("erroroccurred", evtArgs);
        },

        _onAdRefreshed: function () {
            this.dispatchEvent("adrefreshed");
        },

        _onUpdate: function () {
            // we're updating the media. Cancel any active promises created by this plugin
            this._cancelActiveLoad();
        },

        _cancelActiveLoad: function () {
            if (this._activeLoadPromise) {
                this._activeLoadPromise.cancel();
                this._activeLoadPromise = null;
            }
        },

        _onMediaPlayerLoading: function (e) {
            this._activeLoadPromise = this.playAd();
            if (this._activeLoadPromise) {
                e.detail.setPromise(this._activeLoadPromise);
            }
        },

        playAd: function () {
            if (this.adTags !== null && typeof (this.adTags) !== "undefined") {
                var activeLoadPromise = this._getAdManifest().then(
                    function (adManifest) {
                        try {
                            switch (adManifest.type.toLowerCase()) {
                                case "vmap":
                                    var plugin = this.mediaPlayer.vmapSchedulerPlugin;
                                    return plugin.loadAds(adManifest.uri);
                                    break;
                                case "vast":
                                    var adSource = new Microsoft.PlayerFramework.Js.Advertising.RemoteAdSource();
                                    adSource.type = Microsoft.VideoAdvertising.VastAdPayloadHandler.adType;
                                    adSource.uri = new Windows.Foundation.Uri(adManifest.uri);
                                    adSource.headers.insert("User-Agent", Microsoft.VideoAdvertising.Extensions.defaultUserAgent);

                                    if (this.mediaPlayer.playerState === PlayerFramework.PlayerState.unloaded) {
                                        return this.mediaPlayer.adHandlerPlugin.playAd(adSource);
                                    }
                                    else {
                                        var plugin = this.mediaPlayer.adSchedulerPlugin;
                                        var prerollAd = new PlayerFramework.Advertising.PrerollAdvertisement();
                                        prerollAd.source = adSource;
                                        plugin.advertisements.push(prerollAd);
                                    }
                                    break;
                                case "smartxml":
                                    var plugin = this.mediaPlayer.freeWheelPlugin;
                                    return plugin.loadAds(adManifest.uri);
                                    break;
                                default:
                                    this._onErrorOccurred({ errorMessage: "Invalid source uri type", errorCode: "Other" });
                                    break;
                            }
                        }
                        catch (err) {
                        }
                    }.bind(this),
                    function (e) {
                        this.dispatchEvent("loadfailure", e);
                    }.bind(this)
                    );
                return activeLoadPromise;
            }
        },

        _isAdError: function (ad) {
            if (ad === null || typeof ad !== "object") {
                throw Error("Ad is null");
            }

            if (ad.errorCode === null || typeof ad.errorCode === "undefined" || ad.errorCode === "") {
                return false;
            }

            return true;
        },

        _getAdManifest: function () {
            var self = this;
            var adClient = new MicrosoftAdvertising_MMPPF.MultimediaAdClient();
            var rdrOpts = new MicrosoftAdvertising_MMPPF.RendererOptions();

            var placement = new MicrosoftAdvertising_MMPPF.Placement();
            placement.adUnitId = this.adUnitId;
            placement.applicationId = this.applicationId;

            try {
                if (this.adTags !== null && typeof (this.adTags) !== "undefined") {
                    for (var key in this.adTags) {
                        rdrOpts.addRendererOption(key, this.adTags[key]);
                    }
                }
            }
            catch (err) {
                self._onErrorOccurred({ errorMessage: err, errorCode: "Other" });
            }

            return adClient.getAdAsync(placement, rdrOpts).then(function (ad) {
                if (!self._isAdError(ad)) {
                    self._onAdRefreshed();
                    return { uri: ad.sourceURI, type: ad.sourceURIType };
                }
                else {
                    self._onErrorOccurred({ errorMessage: ad.errorMessage, errorCode: ad.errorCode });
                    throw ad.errorMessage;
                }
            },
            function (error) {
                // AdClient library is designed to never throw thus being in here means something went very wrong.
                self._onErrorOccurred({ errorMessage: err, errorCode: "Other" });
            });
        }
    });

    // AdSdkPlugin Mixins
    WinJS.Class.mix(PlayerFramework.MediaPlayer, {
        adSDKPlugin: {
            value: null,
            writable: true,
            configurable: true,
            adUnitId: null,
            applicationId: null,
            adTags: {}
        }
    });

    WinJS.Class.mix(AdSDKPlugin, PlayerFramework.Utilities.createEventProperties(events));

    // AdSdkPlugin Exports
    WinJS.Namespace.define("PlayerFramework.Plugins", {
        AdSDKPlugin: AdSDKPlugin
    });

})(PlayerFramework);