﻿WinJS.Namespace.define('CommonJS', { Version: 'latest' });
/* File: GridClusterView.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexCommonControlsGridClusterViewInit () {
"use strict";
var NV = CommonJS.NavigableView;
var CommonJSNS = WinJS.Namespace.define("CommonJS",
{
/// <summary>
/// A view that lays out its items in a grid. Each item is rendered as a module. Each module
/// can be rendered using many templates. The decision for which templates are applicable to
/// which item is provided as viewOptions to the render function. The contract for the view are
/// the render() and getItemForElement() functions and any other view in the future should
/// implement them.
/// </summary>
GridClusterView: WinJS.Class.define(
/// <summary>
/// Constructor.
/// </summary>
/// <param name="options">
/// The 'mode' property is either CommonJS.GridClusterView.Mode.Uniform or Dynamic. The other properties vary based on the mode:
/// Options for the given mode. The following options are available:
/// 1. For Mode.Uniform: {
///             template: {
///                 id: template ID,
///                 fragmentPath: path to fragment
///             },
///             maxWidth: number (optional, default = unlimited),
///             itemClassName: name of the class to apply to the root element of each item (optional)
///             ads: data about ads. null = no ads
///        }
/// 2. For Mode.Dynamic: {
///             templateSelector: function that maps item to array of templates,
///             alignBottom: true/false (optional, default = false),
///             descendingSizes: true/false, (optional, default = false),
///             maxWidth: number (optional, default = unlimited),
///             itemClassName: name of the class to apply to the root element of each item (optional)
///             ads: data about ads. null = no ads
///        }
/// </param>
function gridClusterView_ctor (options) {
this._options = options;
},
{
/// <summary>
/// Options for the view.
/// </summary>
_options: null,
render: function render(rootElement, data) {
/// <summary>
/// Renders the view and adds it to the DOM.
/// </summary>
/// <param name="rootElement">
/// Root element of the view in the DOM. Assumes the root element is already added to the document and has a height.
/// </param>
/// <param name="data">
/// JSON object that has the following property:
/// - items: Array of items to render.
/// - renderInfos: Metadata about the layout of each item.
/// </param>
/// <returns>
/// A promise that is fulfilled when the rendering is finished.
/// </returns>
var that = this;
var renderInfos = data.renderInfos;
var items = data.items;
var templatesList = [];
var templateInfos = {};
var itemInfos = [];
var arrangeState = {};
var containerElement = document.createElement("div");
var cluster = PlatformJS.Utilities.getAncestorByClassName(rootElement, "platformCluster");
if (cluster) {
var clusterLabelledBy = cluster.getAttribute("aria-labelledby");
if (clusterLabelledBy !== "") {
containerElement.setAttribute("role", "listbox");
containerElement.setAttribute("aria-labelledby", clusterLabelledBy);
}
}
var renderPromise;
if (!items || !items.length) {
renderPromise = WinJS.Promise.wrap({
elements: null
});
} else {
// 1. Prepare - Hits the DOM
renderPromise = that._prepareArrange(rootElement, arrangeState, containerElement)
.then(function gridClusterView_prepareArrangeComplete () {
// 2. Measure templates - Needs the document object
return that._measureTemplates(items, templatesList, templateInfos, renderInfos);
})
.then(function gridClusterView_measureTemplatesComplete () {
// 3. Arrange items - Purely computational, can be pushed off to a worker thread
var isLtr = window.getComputedStyle(document.body).direction !== "rtl";
return that._arrangeItems(items, templatesList, templateInfos, renderInfos, itemInfos, arrangeState, isLtr);
})
.then(function gridClusterView_arrangeItemsComplete () {
// 4. Render items - Hits the DOM
return that._renderItems(itemInfos, containerElement);
})
.then(function gridClusterView_renderItemsComplete (elements) {
// 5. Create render meta data for next time
rootElement.appendChild(containerElement);
var renderInfosThen = that._generateRenderInfos(itemInfos);
return {
elements: elements,
renderInfos: renderInfosThen
};
});
}
return renderPromise;
},
_generateRenderInfos: function _generateRenderInfos(itemInfos) {
/// <summary>
/// Given an array of itemInfos generated in the "Arrange Items" step, creates an array of renderInfos that can be used
/// to re-render this cluster in the same state.
/// </summary>
var renderInfos = [];
for (var i = 0, len = itemInfos.length; i < len; i++) {
var itemInfo = itemInfos[i];
var renderInfo = {
templateInfo: itemInfo.templateInfo,
cellRange: itemInfo.cellRange,
isAd: itemInfo.isAd
};
renderInfos.push(renderInfo);
}
return renderInfos;
},
getItemForElement: function getItemForElement(elt) {
/// <summary>
/// For a given dom element, returns the item context.
/// </summary>
var currentElt = elt;
var item = null;
// walk up the DOM until the module control is found, and get the item from it.
while (currentElt) {
if (WinJS.Utilities.hasClass(currentElt, "cluster-item")) {
item = currentElt.winControl.data;
break;
}
currentElt = currentElt.parentElement;
}
return item;
},
_prepareArrange: function _prepareArrange(rootElement, arrangeState, containerElement) {
/// <summary>
/// Prepare for arranging items.
/// </summary>
// set up root element
WinJS.Utilities.addClass(containerElement, "gridCluster");
rootElement.innerHTML = "";
var height = WinJS.Utilities.getContentHeight(rootElement);
var gridClusterView = CommonJS.GridClusterView;
var rowCount = Math.floor(height / gridClusterView.RowHeight);
if (rowCount > 0) {
containerElement.style.msGridRows = ["(", gridClusterView.RowHeight, "px)[", rowCount, "]"].join("");
// set up cells
var cells = [];
for (var i = 0, len = rowCount; i < len; i++) {
var cell = {
row: i,
column: 0,
moduleWidth: 0,
moduleHeight: 0,
isAvailable: true
};
cells.push([cell]);
}
arrangeState.currentExpandCell = null;
arrangeState.nextExpandCell = cells[0][0];
arrangeState.showAds = this._options.ads || false;
arrangeState.stopArranging = false;
arrangeState.groups = {};
arrangeState.rowCount = rowCount;
arrangeState.cells = cells;
}
return WinJS.Promise.wrap(null);
},
_measureTemplates: function _measureTemplates(items, templatesList, templateInfos, renderInfos) {
/// <summary>
/// Measures all templates and returns a promise whose fulfilled data is a dictionary of templateInfo object, one for each measured template.
/// A template info object has the original template object, plus a height and width. templatesList is a list of list of templates. The
/// element at index N of this list is the list of templates for item N. templatesList is initially empty, and at the end of this function
/// it is fully populated.
/// </summary>
var that = this;
return new WinJS.Promise(function gridClusterView_measureTemplatesPromiseInit (completeFn) {
var options = that._options;
var templateInfoPromises = [];
var processedTemplates = {};
var isRenderInfosValid = renderInfos;
var templateSelectorPromises = [];
for (var j = 0, lenj = items.length; j < lenj; j++) {
var item = items[j];
// use the template selector to get the template
var modes = CommonJS.GridClusterView.Mode;
var mode = options.mode;
if (mode === modes.Dynamic) {
templateSelectorPromises.push(options.templateSelector(item));
} else if (mode === modes.Uniform) {
templateSelectorPromises.push(WinJS.Promise.wrap([options.template]));
}
}
WinJS.Promise.join(templateSelectorPromises).then(function gridClusterView_templateSelectorPromisesJoinComplete (templatesValues) {
for (var i = 0, leni = templatesValues.length; i < leni; i++) {
var templates = templatesValues[i];
var template, templateInfoPromise;
if (renderInfos && isRenderInfosValid) {
// If we can use the cached renderInfos array and the cached template is valid (i.e. exists in the templates array returned by
// the template selector, then use the cached template. As soon as the first invalid cached template is found, invalidate the
// rest of the cache.
var len = renderInfos.length;
if (len && i < len) {
var renderInfo = renderInfos[i];
var cachedTemplateInfo = renderInfo.templateInfo;
var cachedTemplate = cachedTemplateInfo.template;
var isCachedTemplateValid = false;
for (var kk = 0, lenkk = templates.length; kk < lenkk; kk++) {
template = templates[kk];
if (that._areTemplatesEqual(template, cachedTemplate)) {
isCachedTemplateValid = true;
break;
}
}
if (isCachedTemplateValid) {
var cachedTemplateHash = that._getTemplateHash(cachedTemplate);
templatesList.push([cachedTemplate]);
templateInfos[cachedTemplateHash] = cachedTemplateInfo;
processedTemplates[cachedTemplateHash] = true;
continue;
} else {
// First invalid cached template found. Invalidate the rest of the cache.
isRenderInfosValid = false;
renderInfos.splice(i, len - i);
}
}
}
templatesList.push(templates);
for (var k = 0, lenk = templates.length; k < lenk; k++) {
template = templates[k];
var templateHash = that._getTemplateHash(template);
if (processedTemplates[templateHash]) {
continue;
}
processedTemplates[templateHash] = true;
templateInfoPromise = that._measureTemplate(template);
if (templateInfoPromise) {
templateInfoPromises.push(templateInfoPromise);
}
}
}
var ads = options.ads;
if (ads) {
template = ads.template;
templateInfoPromise = that._measureTemplate(template);
if (templateInfoPromise) {
templateInfoPromises.push(templateInfoPromise);
}
}
completeFn(WinJS.Promise.join(templateInfoPromises));
});
}).then(function gridClusterView_measureTemplatesPromiseComplete (templateInfosList) {
// convert list of templateInfos to a map of templateInfos.
if (templateInfosList && templateInfosList.length) {
for (var i = 0, len = templateInfosList.length; i < len; i++) {
var templateInfo = templateInfosList[i];
if (templateInfo) {
templateInfos[that._getTemplateHash(templateInfo.template)] = templateInfo;
}
}
}
});
},
_areTemplatesEqual: function _areTemplatesEqual(template1, template2) {
/// <summary>
/// Returns true if template1 and template2 point to the same template.
/// </summary>
return template1 === null && template2 === null ||
(template1.fragmentPath === template2.fragmentPath && template1.id === template2.id);
},
_getTemplateHash: function _getTemplateHash(template) {
/// <summary>
/// Generates a unique hash for the template based on its fragmentPath and id.
/// </summary>
var templateHash = null;
if (template) {
templateHash = [template.fragmentPath, "#", template.id].join("");
}
return templateHash;
},
_measureTemplate: function _measureTemplate(template) {
/// <summary>
/// Returns the promise of an object that has the width and height of the given template. Null if the template cannot be found.
/// </summary>
return new WinJS.Promise(function gridClusterView_measureTemplatePromiseInit (complete, error, progress) {
WinJS.UI.Fragments.renderCopy(template.fragmentPath)
.then(function gridClusterView_measureTemplateRenderCopyComplete (frag) {
var templateElt = WinJS.Utilities.query("#" + template.id, frag);
var templateInfo = null;
if (templateElt) {
var width = parseInt(templateElt.getAttribute("data-module-width") || 1);
var height = parseInt(templateElt.getAttribute("data-module-height") || 1);
var group = templateElt.getAttribute("data-group") || null;
var bypassDescendingSizes = templateElt.getAttribute("data-bypass-descending-sizes") === "true";
var disablePointerUpAnimations = templateElt.getAttribute("data-disable-pointerup-animations") === "true";
var exclusive = templateElt.getAttribute("data-exclusive") === "true";
templateInfo = {
template: template,
width: width,
height: height,
group: group,
bypassDescendingSizes: bypassDescendingSizes,
disablePointerUpAnimations: disablePointerUpAnimations,
exclusive: exclusive
};
}
complete(templateInfo);
},
function gridClusterView_measureTemplateRenderCopyError () { // on error
complete();
});
});
},
_arrangeItems: function _arrangeItems(items, templatesList, templateInfos, renderInfos, itemInfos, arrangeState, isLtr) {
/// <summary>
/// Arranges items sequentially. Returns a promise that is fulfilled when all items are arranged.
/// </summary>
var ads = this._options.ads;
for (var i = 0, len = items.length; i < len; i++) {
var item = items[i];
var renderInfo = null;
if (ads && renderInfos && i < renderInfos.length) {
renderInfo = renderInfos[i];
if (renderInfo.isAd) {
this._arrangeAd(arrangeState, renderInfo.cellRange, renderInfo.templateInfo, ads);
i++;
continue;
}
}
var templates = templatesList[i];
this._arrangeItem(arrangeState, item, templates, templateInfos, renderInfo, itemInfos);
if (arrangeState.stopArranging) {
break;
}
}
NV.addPointersMetaDataToItemInfos(itemInfos, isLtr, true); // true signifies this is a grid layout rather than pixel layout
// TODO: Returning an empty promise here because ultimately we want to put this in a webworker
return WinJS.Promise.wrap(null);
},
_arrangeAd: function _arrangeAd(arrangeState, cellRange, templateInfo, ads) {
var itemInfo = null;
var gridClusterView = CommonJS.GridClusterView;
// have infinite width to work with, but if ad height doesn't fit don't render it
var availableHeight = cellRange.height;
var height = templateInfo.height;
if (height <= availableHeight) {
var item = ads.adsConfig;
itemInfo = {
templateInfo: templateInfo,
cellRange: cellRange,
item: item,
isAd: true,
bounds: this._getGridItemBounds(cellRange, templateInfo),
pointers: Object.create(NV.ItemPointers)
};
}
return itemInfo;
},
_maybeArrangeAd: function _maybeArrangeAd(arrangeState, templateInfos, itemInfos) {
if (arrangeState.showAds && !arrangeState.stopArranging) {
var cellRange = this._getNextAvailableCellRange(arrangeState);
if (cellRange) {
var column = cellRange.column;
var row = cellRange.row;
var options = this._options;
var maxWidth = options.maxWidth || Number.MAX_VALUE;
var ads = options.ads;
var minAdColumn = ads.minAdColumn;
if (column >= minAdColumn && column < maxWidth && row === 0) {
arrangeState.showAds = false;
var templateInfo = templateInfos[this._getTemplateHash(ads.template)];
var itemInfo = this._arrangeAd(arrangeState, cellRange, templateInfo, ads);
itemInfos.push(itemInfo);
this._addItemToGrid(arrangeState, cellRange, templateInfo);
}
}
}
},
_arrangeItem: function _arrangeItem(arrangeState, item, templates, templateInfos, renderInfo, itemInfos) {
/// <summary>
/// This function measures an item using its pool of templates. templateInfos is a list of all templates and their width
/// and height. This function will attempt to put item in the next column if no suitable template is found to arrange the item.
/// </summary>
// try to render the item
var arranged = this._tryArrangeItem(arrangeState, item, templates, templateInfos, renderInfo, itemInfos);
if (!arranged && !arrangeState.stopArranging) {
// no suitable template found for remaining space, expand to next column and try to render again
var oldExpandCell = arrangeState.currentExpandCell;
arrangeState.currentExpandCell = arrangeState.nextExpandCell;
arranged = this._tryArrangeItem(arrangeState, item, templates, templateInfos, renderInfo, itemInfos);
if (!arranged) {
// if still can't render item, skip rendering the item and go back to the old expand cell
arrangeState.currentExpandCell = oldExpandCell;
}
}
},
_tryArrangeItem: function _tryArrangeItem(arrangeState, item, templates, templateInfos, renderInfo, itemInfos) {
var arranged = false;
// get the next available cell range
var cellRange;
var templateInfo;
if (renderInfo) {
cellRange = renderInfo.cellRange;
templateInfo = renderInfo.templateInfo;
} else {
// try to render the ad first
this._maybeArrangeAd(arrangeState, templateInfos, itemInfos);
cellRange = this._getNextAvailableCellRange(arrangeState);
if (cellRange && !arrangeState.stopArranging) {
var row = cellRange.row;
var column = cellRange.column;
var width = cellRange.width;
var height = cellRange.height;
var gridClusterView = CommonJS.GridClusterView;
var availableWidth = (column === 0 ? width : width - gridClusterView.ColumnMargin);
var availableHeight = (row === 0 ? height : height - gridClusterView.RowMargin);
if (this._options.mode === CommonJS.GridClusterView.Mode.Dynamic && this._options.descendingSizes) {
if (row > 0) {
var previousRowCell = arrangeState.cells[row - 1][column];
if (!previousRowCell.isAvailable && !previousRowCell.bypassDescendingSizes) {
availableWidth = Math.min(availableWidth, previousRowCell.moduleWidth);
availableHeight = Math.min(availableHeight, previousRowCell.moduleHeight);
}
}
}
// make a copy of the array, so we don't modify the original
var templatesCopy = [];
for (var i = 0, len = templates.length; i < len; i++) {
templatesCopy.push(templates[i]);
}
var exclusiveTemplateInfos = [];
while (!templateInfo && templatesCopy.length > 0) {
// randomly pick a template from the pool of templates applicable to the item, until one fits within the availabe height and width
var index = Math.floor(Math.random() * templatesCopy.length); // random index
var template = templatesCopy[index];
var currentTemplateInfo = templateInfos[this._getTemplateHash(template)];
var useTemplate = false;
if (currentTemplateInfo) {
// A template is "group-eligible" if there is no data-group attribute on the template, or the value of the group in _groups is 0.
// When a template is not group-eliglble, it means an item in the same group was rendered in less than N columns before it.
// Groups are used to control the density of certain, often large, templates.
var isGroupEligible = false;
var group = currentTemplateInfo.group;
var groupValue = arrangeState.groups[group];
if (!groupValue) {
isGroupEligible = true;
}
useTemplate =
isGroupEligible &&
(currentTemplateInfo.height <= availableHeight) &&
(currentTemplateInfo.width <= availableWidth);
}
if (useTemplate) {
var exclusive = currentTemplateInfo.exclusive;
if (exclusive) {
exclusiveTemplateInfos.push(currentTemplateInfo);
} else {
templateInfo = currentTemplateInfo;
break;
}
}
// remove template from pool and try again
templatesCopy.splice(index, 1);
}
var exclusiveCount = exclusiveTemplateInfos.length;
if (!templateInfo && exclusiveCount > 0) {
var randomIndex = Math.floor(Math.random() * exclusiveCount);
templateInfo = exclusiveTemplateInfos[randomIndex];
}
}
}
if (templateInfo) {
var itemInfo = {
templateInfo: templateInfo,
cellRange: cellRange,
item: item,
isAd: false,
id: NV.createItemId(),
bounds: this._getGridItemBounds(cellRange, templateInfo),
pointers: Object.create(NV.ItemPointers)
};
this._addItemToGrid(arrangeState, cellRange, templateInfo);
itemInfos.push(itemInfo);
arranged = true;
}
return arranged;
},
_renderItems: function _renderItems(itemInfos, rootElement) {
/// <summary>
/// Render pass - Creates DOM elements, renders templates, and adds them to the DOM.
/// </summary>
var renderPromises = [];
var maxColumn = 0;
for (var i = 0, len = itemInfos.length; i < len; i++) {
var itemInfo = itemInfos[i];
var renderPromise = this._renderItem(itemInfo, rootElement);
renderPromises.push(renderPromise);
var cellRange = itemInfo.cellRange;
maxColumn = Math.max(maxColumn, cellRange.column + itemInfo.templateInfo.width);
}
rootElement.style.msGridColumns = ["(", CommonJS.GridClusterView.ColumnWidth, "px)[", maxColumn, "]"].join("");
return WinJS.Promise
.join(renderPromises)
.then(function gridClusterView_renderPromisesJoinComplete (elements) {
var navigableView = new NV.PointerNavigableView(rootElement);
var rovingTabIndexManager = new NV.RovingTabIndexManager(navigableView);
return elements;
});
},
_renderItem: function _renderItem(itemInfo, rootElement) {
var gridClusterView = CommonJS.GridClusterView;
var templateInfo = itemInfo.templateInfo;
var cellRange = itemInfo.cellRange;
var pointers = itemInfo.pointers;
var item = itemInfo.item;
var isAd = itemInfo.isAd;
var row = cellRange.row;
var column = cellRange.column;
var width = templateInfo.width;
var height = templateInfo.height;
var moduleWidth = templateInfo.width * gridClusterView.ColumnWidth;
var moduleHeight = templateInfo.height * gridClusterView.RowHeight;
// create module elt
var moduleElt = document.createElement("div");
moduleElt.style.width = moduleWidth + "px";
moduleElt.style.height = moduleHeight + "px";
if (!isAd) {
WinJS.Utilities.addClass(moduleElt, "cluster-item");
moduleElt.setAttribute("role", "option");
moduleElt.tabIndex = 0;
moduleElt.id = itemInfo.id;
moduleElt.setAttribute("data-platform-pnv", JSON.stringify(itemInfo.pointers));
// apply root class name
var itemClassName = this._options.itemClassName;
if (itemClassName) {
WinJS.Utilities.addClass(moduleElt, itemClassName);
}
}
if (!templateInfo.disablePointerUpAnimations) {
PlatformJS.Utilities.enablePointerUpDownAnimations(moduleElt);
}
moduleElt.style.msGridRow = row + 1 + (row === 0 ? 0 : gridClusterView.RowMargin);
moduleElt.style.msGridColumn = column + 1 + (column === 0 ? 0 : gridClusterView.ColumnMargin);
moduleElt.style.msGridRowSpan = height;
moduleElt.style.msGridColumnSpan = width;
rootElement.appendChild(moduleElt);
return WinJS.Promise
.timeout()
.then(function gridClusterView_renderItemTimeoutComplete () {
var moduleCtrl = new CommonJS.ModuleControl(moduleElt, {
oneTimeBinding: true,
moduleInfo: {
fragmentPath: templateInfo.template.fragmentPath,
templateId: templateInfo.template.id
},
data: item
});
return moduleCtrl.renderPromise;
});
},
_getNextAvailableCellRange: function _getNextAvailableCellRange(arrangeState) {
/// <summary>
/// Returns an object that represents a range of cells. This object has the following properties:
///     row: Row number of the top left cell in the range (0-based)
///     column: Column number of the top left cell in the range (0-based)
///     width: Number of columns in the range
///     height: Number of rows in the range
/// </summary>
var cellRange = null;
var nextCell = null;
var cells = arrangeState.cells;
var rowCount = arrangeState.rowCount;
var nextExpandCell = arrangeState.nextExpandCell;
var currentExpandCell = arrangeState.currentExpandCell;
if (currentExpandCell) {
for (var i = 0, leni = rowCount; i < leni; i++) {
var currentCellRow = cells[i];
for (var j = currentExpandCell.column, lenj = nextExpandCell.column; j < lenj; j++) {
var cell = currentCellRow[j];
if (cell.isAvailable) {
nextCell = cell;
break;
}
}
if (nextCell) {
break;
}
}
}
// randomize bottom alignment
if (this._options.mode === CommonJS.GridClusterView.Mode.Dynamic &&
!this._options.alignBottom &&
nextCell &&
nextCell.row > Math.ceil(rowCount * 0.8) &&
Math.random() > 0.5) {
nextCell = null;
}
if (nextCell) {
// calculate width
var width = 0;
var cellRow = cells[nextCell.row];
var previousRow = cells[nextCell.row - 1]; // assuming expand cells are in row 0, nextCell.row will always be >= 1
for (var k = nextCell.column, lenk = nextExpandCell.column; k < lenk; k++) {
if (cellRow[k].isAvailable && !previousRow[k].isAvailable) {
width++;
} else {
break;
}
}
// calculate height
var height = rowCount - nextCell.row;
cellRange = {
row: nextCell.row,
column: nextCell.column,
width: width,
height: height
};
} else if (nextExpandCell) {
var maxWidth = this._options.maxWidth;
var availableWidth = maxWidth ? maxWidth - nextExpandCell.column : Number.MAX_VALUE;
if (availableWidth <= 0) {
arrangeState.stopArranging = true;
}
cellRange = {
row: 0,
column: nextExpandCell.column,
width: availableWidth,
height: rowCount
};
}
return cellRange;
},
_addItemToGrid: function _addItemToGrid(arrangeState, cellRange, templateInfo) {
/// <summary>
/// Adds the given DOM element to the grid at the specified location. Also marks the occupied cells as unavailable.
/// If the specified location to add the cell is the nextExpand location, grows the grid.
/// </summary>
var cells = arrangeState.cells;
var nextExpandCell = arrangeState.nextExpandCell;
var currentExpandCell = arrangeState.currentExpandCell;
var groups = arrangeState.groups;
var rowCount = arrangeState.rowCount;
var rootElement = arrangeState.rootElement;
var row = cellRange.row;
var column = cellRange.column;
var gridClusterView = CommonJS.GridClusterView;
var cell = cells[row][column];
var width = templateInfo.width;
var height = templateInfo.height;
var group = templateInfo.group;
var bypassDescendingSizes = templateInfo.bypassDescendingSizes;
var i, j, leni, lenj;
// add columns if necessary
if (cell === nextExpandCell) {
currentExpandCell = arrangeState.currentExpandCell = nextExpandCell;
leni = rowCount;
lenj = currentExpandCell.column + width + 1 + (column === 0 ? 0 : gridClusterView.ColumnMargin);
var startColumn = currentExpandCell.column + 1;
for (i = 0; i < leni; i++) {
for (j = startColumn; j < lenj; j++) {
var newCell = {
row: i,
column: j,
moduleWidth: 0,
moduleHeight: 0,
isAvailable: true
};
cells[i].push(newCell);
}
}
nextExpandCell = arrangeState.nextExpandCell = cells[0][currentExpandCell.column + width + (column === 0 ? 0 : gridClusterView.ColumnMargin)];
// Decrement all group counts since we added a new column
for (var key in groups) {
groups[key] = Math.max(0, groups[key] - 1);
}
}
// mark cells
leni = (row === 0 ? row + height : row + height + gridClusterView.RowMargin);
lenj = (column === 0 ? column + width : column + width + gridClusterView.ColumnMargin);
for (i = row; i < leni; i++) {
var rowi = cells[i];
for (j = column; j < lenj; j++) {
var cellToMark = rowi[j];
cellToMark.isAvailable = false;
cellToMark.moduleWidth = width;
cellToMark.moduleHeight = height;
cellToMark.bypassDescendingSizes = bypassDescendingSizes;
}
}
// mark group
if (group) {
groups[group] = 4; // This means a template in the same group cannot appear more than once every 4 columns
}
},
_getGridItemBounds: function _getGridItemBounds(cellRange, templateInfo) {
return Object.create(NV.ItemBounds, {
top: { value: cellRange.row },
right: { value: cellRange.column + templateInfo.width },
bottom: { value: cellRange.row + templateInfo.height },
left: { value: cellRange.column }
});
}
},
{
Mode: {
Uniform: "Uniform",
Dynamic: "Dynamic"
},
RowMargin: 1,
ColumnMargin: 1,
RowHeight: 10,
ColumnWidth: 10
})
});
})();
/* File: NewsCluster.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexCommonControlsNewsClusterInit (WinJS) {
"use strict";
var RENDER_COMPLETE = "rendercomplete";
var ERROR = "error";
// helper methods
function checkTitle(item, maxLength, minLength) {
return item.title && item.title.length <= maxLength && item.title.length >= minLength;
}
function addTemplate(templates, condition, id) {
if (condition) {
templates.push({
fragmentPath: "/Common/NewsCluster/html/NewsModuleTemplates.html",
id: id
});
}
}
function addTemplates(templates, item, isFirst, hasImage, isLargeSquareImage, isLargeLandscapeImage) {
addTemplate(templates, isFirst && hasImage && checkTitle(item, 60, 30), "largeLandscape_SmallImage");
addTemplate(templates, isFirst, "mediumPortrait_NoImage");
addTemplate(templates, !isFirst && hasImage, "miniModuleB_Image");
addTemplate(templates, !isFirst && hasImage, "mediumPortrait_SmallImage");
addTemplate(templates, !isFirst && hasImage && checkTitle(item, 60, 30), "largeLandscape_SmallImage");
addTemplate(templates, !isFirst, "miniModuleA_NoImage");
addTemplate(templates, !isFirst, "mediumPortrait_NoImagePlain");
addTemplate(templates, !isFirst && checkTitle(item, 60, 30), "mediumPortrait_NoImage");
addTemplate(templates, isLargeSquareImage, "largePortrait_SquareImage");
addTemplate(templates, isLargeLandscapeImage, "largePortrait_LandscapeImage");
addTemplate(templates, checkTitle(item, 60, 30) && isLargeLandscapeImage, "largeLandscape_LandscapeImage");
}
function addEditorialTemplates(templates, item, templateClass, templateMap, fragmentPath, defaultTemplate) {
var isLead = item.isLead;
var template;
if (isLead) {
var leadTemplates = templateClass.getList("Lead");
if (leadTemplates) {
// lead templates specified for class, use them
for (var i = 0, len = leadTemplates.size; i < len; i++) {
var leadTemplate = leadTemplates[i].value;
template = templateMap.getString(leadTemplate);
if (template) {
// lead template recognized
templates.push({
fragmentPath: fragmentPath,
id: template
});
}
}
}
}
if (templates.length === 0) {
// lead templates did not yield any results, use non lead templates instead
var nonLeadTemplates = templateClass.getList("NonLead");
if (nonLeadTemplates) {
// non lead templates specified for class, use them
for (var j = 0, lenj = nonLeadTemplates.size; j < lenj; j++) {
var nonLeadTemplate = nonLeadTemplates[j].value;
template = templateMap.getString(nonLeadTemplate);
if (template) {
// non lead template recognized
templates.push({
fragmentPath: fragmentPath,
id: template
});
}
}
}
}
if (templates.length === 0) {
// no templates in class were recognized, use default
templates.push({
fragmentPath: fragmentPath,
id: defaultTemplate
});
}
}
var News = WinJS.Namespace.define("CommonJS.News",
{
NewsParser: {
parseNewsResponse: function parseNewsResponse(response) {
var resultSets = this._extractOrNull(response, ["BdiGeneric_BingResponse_1_0", "Responses", "0", "ResultSet"]);
var topStories = [];
var newsResults = [];
if (resultSets) {
var mapping = {
Headline: topStories,
TopStories: topStories,
Category: newsResults,
Main: newsResults
};
for (var i = 0, ilen = resultSets.length; i < ilen; i++) {
var resultSet = resultSets[i];
var type = resultSet.moduleName;
if (mapping[type]) {
this._extractNewsResultsToList(resultSet.Results, mapping[type]);
}
}
}
// TODO: figure out what to do with topStories results (edesfos)
// [BEGIN]
return {
native: true,
newsItems: newsResults
};
// [END]
},
_extractOrNull: function _extractOrNull(instance, properties) {
var selected = instance;
var index = 0;
var len = properties.length;
while (selected && index < len) {
var property = properties[index];
selected = selected[property];
index++;
}
return selected;
},
_extractNewsResultsToList: function _extractNewsResultsToList(resultSet, list) {
for (var i = 0, ilen = resultSet.length; i < ilen; i++) {
var article = null;
var image = null;
var result = resultSet[i];
if (result.Type === "Article") {
article = result.Article;
image = article.Image;
} else if (result.Type === "Cluster") {
var cluster = result.Cluster;
article = cluster.Articles[0];
image = cluster.Images ? cluster.Images[0] : null;
}
if (article) {
var parsedArticle = this._parseArticle(article, image);
parsedArticle.index = list.length;
list.push(parsedArticle);
}
}
},
_parseArticle: function _parseArticle(article, image) {
var title = this._removeSearchMarkers(article.Title);
return {
articleUrl: article.Url.Url,
title: title,
category: "",
snippet: this._removeSearchMarkers(article.Snippet),
source: article.Source,
sourceImageUrl: "",
byline: "",
publishTime: CommonJS.Utils.convertBDITimeToFriendlyTime(article.PublishTime.UTCTime),
thumbnail: {
url: image ? "http://www.bing.com" + image.Thumbnail.Url.Url : "",
height: 100,
width: 100,
altText: title,
attribution: ""
}
};
},
_removeSearchMarkers: function _removeSearchMarkers(string) {
var output = [];
for (var i = 0, ilen = string.length; i < ilen; i++) {
var code = string.charCodeAt(i);
if (code !== 57344 && code !== 57345) {
var char = string[i];
output.push(char);
}
}
string = output.join("");
return string;
},
},
NewsProvider: WinJS.Class.define(function newsProvider_ctor () {
//ctor
}, {
// TODO: read from configuration (edesfos)
// [BEGIN]
_applicationIdentifier: "C98EA5B0842DBB9405BBF071E1DA76511BB2CFD7",
// [END]
_queryServiceIdentifier: null,
initializeAsync: function initializeAsync(configuration) {
var identifier = configuration["queryServiceId"];
this._queryServiceIdentifier = identifier;
return WinJS.Promise.wrap(identifier);
},
getQueryServiceOptions: function getQueryServiceOptions() {
var options = new Platform.DataServices.QueryServiceOptions();
options.useCompatDataModel = true;
return options;
},
queryAsync: function queryAsync(query) {
var that = this;
var queryService = new Platform.QueryService(this._queryServiceIdentifier);
var queryParams = PlatformJS.Collections.createStringDictionary();
queryParams.insert("appid", this._applicationIdentifier);
queryParams.insert("query", query["query"]);
queryParams.insert("offset", query["offset"]);
queryParams.insert("count", query["count"]);
queryParams.insert("sortbydate", query["sortByDate"]);
var options = this.getQueryServiceOptions();
return new WinJS.Promise(function newsProvider_queryAsyncPromiseInit (complete, error, progress) {
queryService.downloadDataAsync(queryParams, null, null, options).then(
function newsProvider_downloadDataAsyncComplete (response) {
var data = null;
if (response) {
var jsonResponseData = JSON.parse(response.dataString);
data = News.NewsParser.parseNewsResponse(jsonResponseData);
}
complete(data);
},
function newsProvider_downloadDataAsyncError (e) {
if (PlatformJS.Utilities.isPromiseCanceled(e)) {
return;
}
error(e);
},
function newsProvider_downloadDataAsyncProgress (e) {
if (e.statusCode === Platform.DataServices.QueryServiceStatusCode.cachedEntryFound) {
var data = null;
if (e.cachedResponse) {
var jsonResponseData = JSON.parse(e.cachedResponse.dataString);
data = News.NewsParser.parseNewsResponse(jsonResponseData);
}
progress(data);
}
});
});
}
}),
ClusterMode: {
static: "static",
dynamic: "dynamic"
},
/// <summary>
/// Represents an individual News module in the News cluster.
/// </summary>
NewsModule: WinJS.Class.define(function newsModule_ctor (domElement, options) {
domElement.winControl = this;
this._domElement = domElement;
},
{
/// <summary>
/// Boolean representing whether the module is disabled or not. The class "disabled" is appended to a disabled module.
/// It is up to the control user.
/// </summary>
disabled: {
set: function set(value) {
if (value) {
WinJS.Utilities.addClass(this._domElement, "disabled");
} else {
WinJS.Utilities.removeClass(this._domElement, "disabled");
}
}
}
}),
/// <summary>
/// A Cluster with modules that represent news items. The items in the cluster conform
/// to AppEx.Common.NewsCluster.Response.NewsDescriptor. The module templates for the view assume this interface.
/// </summary>
NewsCluster: WinJS.Class.mix(WinJS.Class.define(function newsCluster_ctor (domElement, options) {
// ctor
var that = this;
this._clearState();
domElement.winControl = this;
this._domElement = domElement;
WinJS.Utilities.addClass(domElement, options.theme || "");
// TODO: move both predicate and completion handler as utility functions for common controls (northamerica\edesfos)
// [BEGIN]
// register itemclick event
Object.defineProperties(this, WinJS.Utilities.createEventProperties("itemclick"));
PlatformJS.Utilities.registerItemClickProxy(domElement,
// predicate
function newsCluster_registerItemClickProxyPredicate (element) {
return WinJS.Utilities.hasClass(element, "cluster-item");
},
// completion
function newsCluster_registerItemClickProxyCompletion (element) {
var item = element.winControl.data;
that.dispatchEvent("itemclick", {
item: item
});
});
// [END]
// register rendercomplete event
Object.defineProperties(this, WinJS.Utilities.createEventProperties(RENDER_COMPLETE));
// register error event
Object.defineProperties(this, WinJS.Utilities.createEventProperties(ERROR));
// null check view (options)
var viewInfo = options.viewInfo;
if (!viewInfo) {
viewInfo = {
type: "CommonJS.GridClusterView",
options: {
mode: CommonJS.GridClusterView.Mode.Dynamic,
templateSelector: CommonJS.News.NewsCluster.templateSelector
}
};
}
var unitWidth = (News.NewsCluster.templateUnitWidth / CommonJS.GridClusterView.ColumnWidth);
// set maxWidth on the view if maxColumnCount is provided in options
var maxColumnCount = options.maxColumnCount;
if (maxColumnCount) {
viewInfo.options.maxWidth = maxColumnCount * unitWidth - 1;
}
// override templateSelector if provided in options
var templateSelector = options.templateSelector;
if (templateSelector) {
viewInfo.options.templateSelector = templateSelector;
}
// descendingSizes is always true for news clusters
viewInfo.options.descendingSizes = true;
// override alignBottom if provided in options
var alignBottom = options.alignBottom;
if (typeof(alignBottom) === "boolean") {
viewInfo.options.alignBottom = alignBottom;
} else {
// default is true
viewInfo.options.alignBottom = true;
}
// configure ads
var adsConfig = options.adsConfig;
if (adsConfig) {
var adClusterId = adsConfig.adClusterId;
var platformAdsConfig = null;
if (adClusterId && PlatformJS.Ads) {
platformAdsConfig = PlatformJS.Ads.getAdsConfig(adClusterId);
}
if (platformAdsConfig) {
viewInfo.options.ads = {
minAdColumn: (News.NewsCluster.adColumn - 1) * unitWidth - 1,
template: {
fragmentPath: "/Common/NewsCluster/html/NewsModuleTemplates.html",
id: "tileAd"
},
adsConfig: platformAdsConfig
};
}
}
// create the view
this.view = PlatformJS.Utilities.createObject(viewInfo.type, viewInfo.options);
// null check provider
if (options.mode === News.ClusterMode.dynamic && !options.provider) {
options.provider = new News.NewsProvider();
}
this._currentHeight = domElement.offsetHeight;
domElement.onresize = function (evt) {
// on resize, rerender using new height
var newHeight = domElement.offsetHeight;
var currentHeight = that._currentHeight;
if (newHeight !== currentHeight) {
that._currentHeight = newHeight;
that.render();
}
};
this._progressElt = null;
this._errorElt = null;
// set options
WinJS.UI.setOptions(this, options);
},
{
// declare null aliases for auto-complete to work
_domElement: null,
onitemclick: null,
onrendercomplete: null,
onerror: null,
view: null,
_state: null,
_currentHeight: null,
_progressElt: null,
_errorElt: null,
_newsItems: null,
_uniqueID: null,
newsItems: {
get: function get() {
return this._newsItems;
}
},
state: {
get: function get() {
return this._state;
},
set: function set(value) {
this._state = value;
}
},
getState: function getState() {
/// <summary>Returns the cluster state. Called by the Panorama control.</summary>
return this._state;
},
_clearState: function _clearState() {
this._state = {};
},
_saveState: function _saveState(state) {
var height = this._currentHeight;
this._state[height] = state;
},
_getState: function _getState() {
var height = this._currentHeight;
return this._state[height];
},
/// <summary>
/// Invalidates the News cluster by querying the service and binding the resulting data to the view.
/// </summary>
render: function render() {
var that = this;
this._uniqueID = performance.now();
return new WinJS.Promise(function newsCluster_renderPromiseInit (complete, error, progress) {
msWriteProfilerMark("CommonControls:NewsCluster:render:" + that._uniqueID + ":s");
var getNewsItems;
var mode = that.mode;
if (mode === News.ClusterMode.dynamic) {
getNewsItems = that._queryProviderAsync();
} else {
var dataSet = that.dataSet || [];
getNewsItems = WinJS.Promise.wrap(dataSet);
}
msWriteProfilerMark("CommonControls:NewsCluster:getNewsItems:s");
var p = WinJS.Promise.wrap(null);
var cachedResponseRendered = false;
getNewsItems.then(
function newsCluster_getNewsItemsComplete(response) {
if (!(response.isCachedResponse || response.serverResponseNotModified) || !cachedResponseRendered) {
p = p.then(
function newsCluster_getNewsItemsResponseComplete () {
return that._renderResponse(response);
},
function newsCluster_getNewsItemsResponseError (err) {
that._renderError(err);
});
return p;
} else {
return WinJS.Promise.wrap(null);
}
},
null,
function newsCluster_getNewsItemsProgress (response) {
cachedResponseRendered = true;
p = p.then(
function newsCluster_getNewsItemsProgressComplete () {
return that._renderResponse(response);
}).then(function newsCluster_getNewsItemsProgressError () {
complete();
});
return p;
}
)
.then(
function newsCluster_renderComplete() {
complete();
},
function newsCluster_renderError(err) {
if (!PlatformJS.Utilities.isPromiseCanceled(err)) {
that._renderError(err);
}
complete();
});
});
},
_renderResponse: function _renderResponse(response) {
var that = this;
var view = that.view;
var domElement = that._domElement;
msWriteProfilerMark("CommonControls:NewsCluster:getNewsItems:e");
var newsItems = that._newsItems = (response ? response.newsItems : null);
// get cluster state
var state = that._getState();
var viewData = {
items: newsItems,
renderInfos: state ? state.renderInfos : null
};
return view.render(domElement, viewData)
.then(
function newsCluster_viewRenderComplete (result) {
try {
that._hideProgress();
that._hideError();
var elements = result.elements;
var renderInfos = result.renderInfos;
// save metadata in cluster state
var myState = {
renderInfos: renderInfos
};
that._saveState(myState);
if (elements) {
that._preRenderComplete(elements);
that.dispatchEvent(RENDER_COMPLETE, elements);
}
msWriteProfilerMark("CommonControls:NewsCluster:render:" + that._uniqueID + ":e");
// Marker for test automation
msWriteProfilerMark("CommonControls:NewsCluster:RenderComplete");
}
catch (err) {
that._renderError(err);
}
},
function newsCluster_viewRenderError (err) {
that._renderError(err);
}
);
},
_renderError: function _renderError(err) {
var that = this;
// clear out state on error
that._clearState();
that._showError();
// dispatch error event
that.dispatchEvent(ERROR, err);
// complete render promise
msWriteProfilerMark("CommonControls:NewsCluster:render:" + that._uniqueID + ":e");
// Marker for test automation
msWriteProfilerMark("CommonControls:NewsCluster:RenderComplete");
},
_showError: function _showError() {
this._createErrorModule();
this._hideProgress();
WinJS.Utilities.removeClass(this._errorElt, "platformHide");
},
_hideError: function _hideError() {
if (this._errorElt) {
WinJS.Utilities.addClass(this._errorElt, "platformHide");
}
},
_showProgress: function _showProgress() {
if (!this._progressElt) {
this._createProgressModule();
}
this._hideError();
WinJS.Utilities.removeClass(this._progressElt, "platformHide");
},
_hideProgress: function _hideProgress() {
if (this._progressElt) {
WinJS.Utilities.addClass(this._progressElt, "platformHide");
}
},
_createErrorModule: function _createErrorModule() {
var that = this;
var data;
var retryFunction = WinJS.Utilities.markSupportedForProcessing(function newsCluster_retryFunctionHandler (evt) {
that.render();
});
if (!PlatformJS.Utilities.hasInternetConnection()) {
data = CommonJS.Error.getErrorModuleItem(retryFunction, "450px", "100%", "/platform/offline_noContent", "platformOfflineErrorModule");
} else {
data = CommonJS.Error.getErrorModuleItem(retryFunction, "450px", "100%");
}
var moduleInfo = data.moduleInfo;
var domElement = this._domElement;
var errorElt = document.createElement("div");
errorElt.className = "platformHide";
errorElt.style.width = moduleInfo.width;
errorElt.style.height = moduleInfo.height;
domElement.innerHTML = "";
domElement.appendChild(errorElt);
CommonJS.loadModule(moduleInfo, data, errorElt);
this._errorElt = errorElt;
},
_createProgressModule: function _createProgressModule() {
var data = CommonJS.Error.getProgressModuleItem();
var moduleInfo = data.moduleInfo;
var domElement = this._domElement;
var progressElt = document.createElement("div");
progressElt.className = "platformHide";
progressElt.style.width = moduleInfo.width;
progressElt.style.height = moduleInfo.height;
domElement.appendChild(progressElt);
CommonJS.loadModule(moduleInfo, data, progressElt);
this._progressElt = progressElt;
},
_queryFromRenderedData: function _queryFromRenderedData(renderedData) {
/// <summary>Returns an object that conforms to the news provider return type from the given renderedData object.</summary>
return {
newsItems: renderedData
};
},
_processData: function _processData(queryResult) {
var data = null;
// if response is marshalled from managed, convert
if (queryResult && !queryResult.native) {
data = PlatformJS.Utilities.convertManageToJSON(queryResult);
} else {
data = queryResult;
}
return data;
},
_queryProviderAsync: function _queryProviderAsync() {
var that = this;
this._showProgress();
// start invalidation
var providerConfiguration = PlatformJS.Collections.createStringDictionary();
providerConfiguration.insert("queryServiceId", this.queryServiceId);
return new WinJS.Promise(function newsCluster_queryProviderAsyncPromiseInit (complete, error, progress) {
that.provider.initializeAsync(providerConfiguration)
.then(function newsCluster_providerInitializeAsyncComplete (initResult) {
if (!initResult) {
throw "NewsCluster initialization error.";
}
var query = PlatformJS.Collections.createStringDictionary();
for (var key in that.newsQuery) {
query.insert(key, that.newsQuery[key] + "");
}
return that.provider.queryAsync(query);
})
.then(
function newsCluster_providerQueryAsyncComplete (queryResult) {
var data = that._processData(queryResult);
complete(data);
},
function newsCluster_providerQueryAsyncError (e) {
error(e);
},
function newsCluster_providerQueryAsyncProgress (queryResult) {
var data = that._processData(queryResult);
progress(data);
});
});
},
_preRenderComplete: function _preRenderComplete(elements) {
if (elements) {
for (var i = 0, leni = elements.length; i < leni; i++) {
var element = elements[i];
// handle overflow text
this._handleOverflowText(element);
}
}
},
_handleOverflowText: function _handleOverflowText(element) {
var multilineElts = element.querySelectorAll(".multiline");
var utils = CommonJS.Utils;
for (var j = 0, lenj = multilineElts.length; j < lenj; j++) {
var multilineElt = multilineElts[j];
utils.truncateMultilineText(multilineElt);
}
}
}, {
// begin static type definition
/// <summary>
/// For a given item, returns a list of templates that can be used to render that item.
/// Each template is represented by a JSON object that has two properties:
///     1. fragmentPath: Path to a fragment that contains that template
///     2. id: ID of the template
/// </summary>
/// <param name="item">
/// The item.
/// </param>
templateSelector: function templateSelector(item) {
return new WinJS.Promise(function newsCluster_templateSelectorPromiseInit (complete) {
var templates = [];
var thumbnail = item.thumbnail;
var url = thumbnail ? thumbnail.url : null;
var hasImage = url ? true : false;
var isFirst = (item.index === 0);
var isLargeSquareImage = false;
var isLargeLandscapeImage = false;
if (hasImage) {
var isNetworkAvailable = Platform.Networking.NetworkManager.instance.isNetworkAvailable; // TODO may be an expensive call, move out of templateSelector
if (!isNetworkAvailable) {
var imageService = new Platform.ImageService("PlatformImageCache");
imageService.hasEntryAsync(url).then(
function newsCluster_imageServiceHasEntryAsyncComplete (cachedUrl) {
if (cachedUrl) {
return true;
} else {
return false;
}
},
function newsCluster_imageServiceHasEntryAsyncError () {
// assume no image in error case
return false;
}).then(
function newsCluster_hasEntryComplete (hasImg) {
if (hasImg) {
var aspectRatio = thumbnail.width / thumbnail.height;
if (aspectRatio === 1 && thumbnail.width >= 200) {
isLargeSquareImage = true;
} else if (aspectRatio === 2) {
isLargeLandscapeImage = true;
}
}
addTemplates(templates, item, isFirst, hasImg, isLargeSquareImage, isLargeLandscapeImage);
complete(templates);
});
} else {
addTemplates(templates, item, isFirst, hasImage, isLargeSquareImage, isLargeLandscapeImage);
complete(templates);
}
} else {
addTemplates(templates, item, isFirst, hasImage, isLargeSquareImage, isLargeLandscapeImage);
complete(templates);
}
});
},
editorialTemplateSelector: function editorialTemplateSelector(item) {
// TODO (naeims): In GA, refactor this function to be more be more promise-chain-driven rather than using so many nested conditions and booleans.
return new WinJS.Promise(function newsCluster_editorialTemplateSelectorPromiseInit (complete) {
var fragmentPath = "/Common/NewsCluster/html/EditorialModuleTemplates.html";
var templateMap = PlatformJS.Services.configuration.getDictionary("EditorialTemplateMapping");
var templateClasses = PlatformJS.Services.configuration.getDictionary("EditorialTemplateClasses");
var defaultTemplate = templateMap.getString(PlatformJS.Services.configuration.getString("EditorialDefaultTemplate"));
var itemTemplateClass = item.templateClass;
var useTemplateId = false;
var isWaitingForImageService = false;
var isNetworkAvailable;
if (itemTemplateClass) {
// if template class is specified, respect it
var addDefault = false;
var addEditorial = false;
var templateClass = templateClasses.getDictionary(itemTemplateClass);
if (templateClass.size !== 0) {
// if template class is recognized
var options = templateClass.getDictionary("Options");
var isImageClass = false;
if (options.size !== 0) {
isImageClass = options.getBool("HasImage");
}
if (isImageClass) {
// template class has images
var thumbnail = item.thumbnail;
var url = thumbnail ? thumbnail.url : null;
var hasImage = url ? true : false;
if (hasImage) {
// data has image (if not true, editorial has assigned an image class to data with no images)
isNetworkAvailable = Platform.Networking.NetworkManager.instance.isNetworkAvailable; // TODO may be an expensive call, move out of templateSelector
if (!isNetworkAvailable) {
// if network is not available, check cache for image
var imageService = new Platform.ImageService("PlatformImageCache");
imageService.hasEntryAsync(url)
.then(
function newsCluster_imageServiceHasEntryAsyncComplete (cachedUrl) {
if (cachedUrl) {
return true;
} else {
return false;
}
},
function newsCluster_imageServiceHasEntryAsyncError () {
// assume no image in error case
return false;
})
.then(function newsCluster_hasEntryComplete (hasImg) {
var templates = [];
if (hasImg) {
// if image exists in image cache, proceed as normal
addEditorialTemplates(templates, item, templateClass, templateMap, fragmentPath, defaultTemplate);
complete(templates);
} else {
// if image does not exist in cache, use default template
templates.push({
fragmentPath: fragmentPath,
id: defaultTemplate
});
complete(templates);
}
});
isWaitingForImageService = true;
} else {
// network is available, proceed as normal
addEditorial = true;
}
} else {
// data does not contain image, but image template class chosen, use default template
addDefault = true;
}
} else {
// template is not an image class, proceed as normal
addEditorial = true;
}
} else {
// if template class is not recognized, use legacy template id
useTemplateId = true;
}
if (!useTemplateId && !isWaitingForImageService) {
var templates1 = [];
if (addEditorial) {
addEditorialTemplates(templates1, item, templateClass, templateMap, fragmentPath, defaultTemplate);
} else if (addDefault) {
templates1.push({
fragmentPath: fragmentPath,
id: defaultTemplate
});
}
complete(templates1);
}
} else {
useTemplateId = true;
}
if (useTemplateId && !isWaitingForImageService) {
// use legacy template ID
if (typeof isNetworkAvailable === "undefined" /*didn't have a chance to set it yet*/) {
isNetworkAvailable = Platform.Networking.NetworkManager.instance.isNetworkAvailable;
}
if (!isNetworkAvailable) {
// if network is not available, use default template
var templatesOffline = [];
templatesOffline.push({
fragmentPath: fragmentPath,
id: defaultTemplate
});
complete(templatesOffline);
} else {
// otherwise if network is available, respect the template id
var itemTemplate = item.template;
var template = templateMap.getString(itemTemplate);
if (template) {
// if template is recognized, use it
var templatesSpecific = [];
templatesSpecific.push({
fragmentPath: fragmentPath,
id: template
});
complete(templatesSpecific);
} else {
// otherwise, use the default template
var templatesDefault = [];
templatesDefault.push({
fragmentPath: fragmentPath,
id: defaultTemplate
});
complete(templatesDefault);
}
}
}
});
},
itemClickHandler: function itemClickHandler(evt) {
// launch url in browser
var item = evt.item;
var url = item.articleUrl;
if (url) {
var uri = new Windows.Foundation.Uri(url);
Windows.System.Launcher.launchUriAsync(uri).then();
}
},
sourceImageDisplay: WinJS.Binding.converter(function newsCluster_sourceImageDisplay  (sourceImageUrl) {
return sourceImageUrl ? "block" : "none";
}),
trimSnippet: WinJS.Binding.converter(function newsCluster_trimSnippet (snippet) {
return CommonJS.Utils.truncateString(snippet, 125);
}),
directionAttribute: WinJS.Binding.converter(function newsCluster_directionAttribute (text) {
return PlatformJS.Utilities.getTextDirection(text);
}),
/// <summary>
/// Width of the smallest template in pixels.
/// </summary>
templateUnitWidth: 310,
/// <summary>
/// Column number of the ad
/// </summary>
adColumn: 4
// end static type definition
}), WinJS.UI.DOMEventMixin)
});
})(WinJS);
/* File: RSSNewsProvider.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function () {
"use strict";
WinJS.Namespace.define("CommonJS.NewsRSS",
{
NewsProvider: WinJS.Class.define(function (options) {
this._feed = options.feed;
this._sourceName = options.sourceName;
this._title = options.title;
this._showProgress = options.showProgress;
this._disableFavIcon = options.disableFavIcon;
this._bypassCache = options.bypassCache;
this._noResultsTile = options.noResultsTile;
this._onsuccess = options.onsuccess;
this._onerror = options.onerror;
}, {
_maxArticles: 40,
_articles: null,
_bypassCache: false,
_parser: null,
_clusterID: null,
title: {
get: function get() {
return this._title;
}
},
articles: {
get: function get() {
return this._articles;
}
},
clusterID: {
get: function get() {
return this._clusterID;
},
set: function set(cid) {
this._clusterID = cid;
}
},
initializeAsync: function initializeAsync(configuration) {
return WinJS.Promise.wrap({});
},
queryAsync: function queryAsync(query) {
var that = this;
var queryService = new PlatformJS.DataService.QueryService("RSS");
var urlParams = PlatformJS.Collections.createStringDictionary();
urlParams.insert("url", this._feed);
if (this._showProgress) {
CommonJS.Progress.showProgress(CommonJS.Progress.headerProgressType);
}
var options = new Platform.DataServices.QueryServiceOptions();
options.useExtendedEncoding = true;
options.useCompatDataModel = true;
options.bypassCache = this._bypassCache;
that.layoutCacheInfo = { cacheKey: queryService.getUrl(urlParams), cacheID: queryService.getCacheID() };
return queryService.downloadDataAsync(urlParams, null, null, options).then(function (e) {
if (that.layoutCacheInfo && e) {
that.layoutCacheInfo.uniqueResponseID = e.responseId;
}
if (that._onsuccess) {
that._onsuccess(e);
}
if (that._showProgress) {
CommonJS.Progress.hideProgress(CommonJS.Progress.headerProgressType);
}
that._articles = that._parseRSSDocument(e.dataString);
if (that.articles.length === 0) {
// instrument the failure
var message = "Failed to parse RSS Feed";
var attributes = {
URL: that._feed,
Location: "CommonJS.NewsRSS.NewsProvider"
};
Platform.Telemetry.FlightRecorder.logCodeErrorWithJsonAttributes(
Platform.Telemetry.LogLevel.normal,
Platform.Telemetry.RuntimeEnvironment.javascript,
message,
null,
JSON.stringify(attributes));
}
if (that.articles.length === 0 && that._noResultsTile) {
that.articles.push(that._noResultsTile);
}
return {
native: true,
newsItems: that._articles,
clusterID: that._clusterID,
cacheID: that.layoutCacheInfo.cacheID,
dataCacheID: that.layoutCacheInfo.cacheKey,
uniqueResponseID: that.layoutCacheInfo.uniqueResponseID
};
}, function (error) {
if (that._onerror) {
that._onerror(error);
}
if (that._showProgress) {
CommonJS.Progress.hideProgress(CommonJS.Progress.headerProgressType);
}
if (PlatformJS.Utilities.isPromiseCanceled(error)) {
return;
}
throw error;
});
},
_getElementsByTagNames: function _getElementsByTagNames(dom, tagNames) {
if (typeof tagNames === "string") {
tagNames = [tagNames];
}
for (var i = 0; i < tagNames.length; i++) {
var tagName = tagNames[i];
var elements = dom.getElementsByTagName(tagName);
if (elements && elements.length > 0) {
return elements;
}
}
return null;
},
_getElementByTagNames: function _getElementByTagNames(dom, tagNames) {
var elements = this._getElementsByTagNames(dom, tagNames);
if (elements && elements.length > 0) {
return elements[0];
}
return null;
},
_parseRSSItem: function _parseRSSItem(dom) {
var article = {};
var url, type, height, width;
// set the source name
article.source = this._sourceName;
// grab the publish date and calculate offset
var pubDate = this._getElementByTagNames(dom, ["pubDate", "published"]);
if (pubDate && pubDate.textContent) {
article.publishTime = CommonJS.Utils.convertRSSTimeToFriendlyTime(pubDate.textContent);
} else {
article.publishTime = "";
}
// grab title from feed
article.title = "";
var title = this._getElementByTagNames(dom, "title");
if (title && this._parser) {
try {
var titleDom = this._parser.parseFromString(title.textContent, "text/html");
article.title = this._parseTextNodes(titleDom);
} catch (e) {
}
}
var thumb = this._getElementByTagNames(dom, "media:thumbnail");
if (thumb) {
url = thumb.attributes.getNamedItem("url");
if (url) {
article.thumbnail = {};
article.thumbnail.url = url.textContent;
width = thumb.attributes.getNamedItem("width");
if (width) {
article.thumbnail.width = width.textContent;
}
height = thumb.attributes.getNamedItem("height");
if (height) {
article.thumbnail.height = height.textContent;
}
}
}
else {
var description = this._getElementByTagNames(dom, "description");
if (description) {
var descriptionImages = this._parseAllImageTagsFromDescription(description),
imageWithWidthAndHeight = this._findFirstImageWithWidthAndHeight(descriptionImages);
if (imageWithWidthAndHeight) {
article.thumbnail = imageWithWidthAndHeight;
}
}
}
// look for enclosure
var enclosures = dom.getElementsByTagName("enclosure");
for (var i = 0 ; i < enclosures.length; i++) {
var enclosure = enclosures[i];
url = enclosure.attributes.getNamedItem("url");
type = enclosure.attributes.getNamedItem("type");
if (url && url.textContent && type && type.textContent && type.textContent.startsWith("image/")) {
if (!article.thumbnail) {
article.thumbnail = {};
article.thumbnail.url = url.textContent;
}
break;
}
}
// look for media
var medias = dom.getElementsByTagName("media:content");
for (var k = 0; k < medias.length; k++) {
var media = medias[k];
url = media.attributes.getNamedItem("url");
type = media.attributes.getNamedItem("type");
var medium = media.attributes.getNamedItem("medium");
if (url && (type || medium)) {
if ((type && type.textContent.startsWith("image/")) || (medium && (medium.textContent === "image"))) {
width = media.attributes.getNamedItem("width");
// If there's no width or if width is less than 200, use as thumbnail
if (!width || width.textContent <= 200) {
if (!article.thumbnail) {
article.thumbnail = {};
article.thumbnail.url = url.textContent;
if (width) {
article.thumbnail.width = width.textContent;
}
height = media.attributes.getNamedItem("height");
if (height) {
article.thumbnail.height = height.textContent;
}
}
} else {
// Else, treat it as a big image to use on the story page
article.imageUrl = url.textContent;
}
} else if (type && type.textContent === "video/mp4") {
// Remember video url to use on story page
article.videoUrl = url.textContent;
}
}
}
// look for a snippet
article.snippet = "";
var desc = this._getElementByTagNames(dom, ["content:encoded", "description", "summary", "content"]);
if (desc && this._parser) {
article.html = desc.textContent;
try {
var descDom = this._parser.parseFromString(desc.textContent, "text/html");
article.snippet = this._parseTextNodes(descDom);
} catch (e) {
}
}
// get the link to the full article
var link = null, linkNoRel = null, linkAlternate = null;
var links = dom.getElementsByTagName("link");
// loop through the links and possibly override the default
if (links && links.length) {
for (var j = 0; j < links.length; j++) {
var rel = links[j].attributes.getNamedItem("rel");
if (!linkNoRel && (!rel || !rel.textContent)) {
linkNoRel = links[j];
} else if (!linkAlternate && rel && rel.textContent.toLowerCase() === "alternate") {
linkAlternate = links[j];
}
}
}
if (linkNoRel) {
// give links with no relationship attribute first priority
link = linkNoRel;
} else if (linkAlternate) {
// give links with alternate relationship attribute 2nd priority
link = linkAlternate;
} else if (links && links.length > 0) {
// otherwise, just take the first link
link = links[0];
}
if (link) {
var href = link.attributes.getNamedItem("href");
if (href && href.textContent) {
article.articleUrl = href.textContent;
} else if (link.textContent) {
article.articleUrl = link.textContent;
}
article.sourceImageUrl = "";
if (article.articleUrl && !this._disableFavIcon) {
var uri = new Windows.Foundation.Uri(article.articleUrl);
if (uri) {
article.sourceImageUrl = "http://" + uri.host + "/favicon.ico";
}
}
}
if (!article.articleUrl) {
// If there is no URL to the article, there's no point in showing the article
return null;
}
if (article.thumbnail
&& ((article.thumbnail.height && article.thumbnail.height < 100) || (article.thumbnail.width && article.thumbnail.width < 100))) {
// If the thumbnail is too small, we cue templateClass as "G" which is no image template.
article.templateClass = "G";
}
return article;
},
_parseTextNodesWorker: function _parseTextNodesWorker(node) {
var that = this;
var nodeText = "";
if (node) {
if (node.nodeType === 3) {
nodeText = node.data;
} else if (node.nodeType === 1 && node.childNodes) {
var nodeTagName = node.tagName.toUpperCase();
if (!(nodeTagName === "SCRIPT") && !(nodeTagName === "STYLE")) {
for (var i = 0; i < node.childNodes.length; i++) {
nodeText = nodeText + that._parseTextNodesWorker(node.childNodes[i]);
}
}
}
}
return nodeText;
},
_parseTextNodes: function _parseTextNodes(root) {
return this._parseTextNodesWorker(root.body);
},
_parseRSSDocument: function _parseRSSDocument(response) {
var articles = [];
if (window.DOMParser) {
this._parser = new window.DOMParser();
var xmlDoc = null;
try {
xmlDoc = this._parser.parseFromString(response, "text/xml");
} catch (e) {
xmlDoc = null;
}
if (xmlDoc) {
var items = this._getElementsByTagNames(xmlDoc, ["item", "entry"]);
if (items) {
for (var i = 0; i < items.length && i < this._maxArticles; i++) {
var item = this._parseRSSItem(items[i]);
if (item) {
articles.push(item);
}
}
}
}
}
return articles;
},
_parseAllImageTagsFromDescription: function _parseAllImageTagsFromDescription(descriptionNode) {
var imageTags = [];
try {
var descriptionDom = this._parser.parseFromString(descriptionNode.textContent, "text/html"),
nodes = descriptionDom.images;
// the nodes returned by descriptionDom.images are not all images!
// so we filter out the real images here
for (var index in nodes) {
var node = nodes[index];
if (node.tagName && node.tagName.toLowerCase() === "img") {
imageTags.push(node);
}
}
}
catch (e) {
// do nothing. an empty array will be returned.
}
return imageTags;
},
_findFirstImageWithWidthAndHeight: function _findFirstImageWithWidthAndHeight(imageEls) {
var imagesWithWidthAndHeight = this._findImagesWithWidthAndHeight(imageEls);
return (imagesWithWidthAndHeight.length > 0) ? imagesWithWidthAndHeight[0] : null;
},
_findImagesWithWidthAndHeight: function _findImagesWithWidthAndHeight(imageEls) {
var imagesWithWidthAndHeight = [],
_parseDimensionsFromImageEl = this._parseDimensionsFromImageEl;
imageEls.forEach(function (imageEl) {
var imageDimensions = _parseDimensionsFromImageEl(imageEl);
if (imageDimensions.width && imageDimensions.height) {
imagesWithWidthAndHeight.push({ url: imageEl.src, width: imageDimensions.width, height: imageDimensions.height });
}
});
return imagesWithWidthAndHeight;
},
_parseDimensionsFromImageEl: function _parseDimensionsFromImageEl(imageEl) {
var dimensions = { width: null, height: null };
dimensions.width = parseInt(imageEl.width, 10) || parseInt(imageEl.currentStyle.width, 10);
dimensions.height = parseInt(imageEl.height, 10) || parseInt(imageEl.currentStyle.height, 10);
return dimensions;
}
})
});
})();
/* File: DefaultFinanceSportsConfig.js */
(function () {
"use strict";
WinJS.Namespace.define("CommonJS.News.EntityClusterConfig.DefaultFinanceSports",
{
TemplateClassMap:
{
B: {
classID: "B",
snippetMin: 650,
aspectRatioMin: 1.3,
aspectRatioMax: 1.9,
isDefault: false,
type: "News",
templateList: [
"Large_Landscape_2col_3row",
"Large_Landscape_2col_4row",
"Medium_Landscape_1col_4row",
"Medium_Landscape_1col_3row",
"Medium_Landscape_LongHeadline_1col_3row",
"None_1col_1row"
]
},
A: {
classID: "A",
snippetMin: 0,
aspectRatioMin: 1.3,
aspectRatioMax: 1.9,
isDefault: false,
type: "News",
templateList: [
"Large_Landscape_2col_3row",
"Medium_Landscape_1col_3row",
"Medium_Landscape_LongHeadline_1col_3row",
"None_1col_1row"
]
},
C: {
classID: "C",
snippetMin: 0,
aspectRatioMin: 0.5,
aspectRatioMax: 0.9,
isDefault: false,
type: "News",
templateList: [
"Medium_Portrait_1col_4row",
"None_LongHeadline_1col_3row",
"None_1col_2row",
"None_1col_1row"
]
},
I: {
classID: "I",
snippetMin: 650,
aspectRatioMin: 0.9,
aspectRatioMax: 1.3,
isDefault: false,
type: "News",
templateList: [
"Large_Square_2col_4row",
"Medium_Square_1col_4row",
"Medium_Square_1col_3row",
"Small_Square_1col_2row",
"None_LongHeadline_1col_3row",
"None_1col_1row"
]
},
E: {
classID: "E",
snippetMin: 0,
aspectRatioMin: 0.9,
aspectRatioMax: 1.3,
isDefault: false,
type: "News",
templateList: [
"Large_Square_2col_4row",
"Medium_Square_1col_4row",
"Medium_Square_1col_3row",
"Small_Square_1col_2row",
"None_LongHeadline_1col_3row",
"None_1col_1row"
]
},
F: {
classID: "F",
snippetMin: 0,
aspectRatioMin: 0,
aspectRatioMax: 0,
isDefault: false,
type: "News",
templateList: [
"None_LongHeadline_1col_3row",
"None_1col_2row",
"None_1col_1row"
]
},
H: {
classID: "H",
snippetMin: 650,
aspectRatioMin: 0,
aspectRatioMax: 0,
isDefault: true,
type: "News",
templateList: [
"None_LongHeadline_1col_3row",
"None_1col_2row",
"None_1col_1row",
"None_1col_4row"
]
},
G: {
classID: "G",
snippetMin: 0,
aspectRatioMin: 0,
aspectRatioMax: 0,
isDefault: true,
type: "News",
templateList: [
"None_LongHeadline_1col_3row",
"None_1col_2row",
"None_1col_1row"
]
},
S: {
classID: "S",
snippetMin: 0,
aspectRatioMin: 0,
aspectRatioMax: 0,
isDefault: false,
type: "Slideshow",
templateList: [
"Large_Slideshow_2col_4row",
"Medium_Slideshow_1col_2row",
"Small_Slideshow_1col_1row"
]
},
V: {
classID: "V",
snippetMin: 0,
aspectRatioMin: 0,
aspectRatioMax: 0,
isDefault: false,
type: "Video",
templateList: [
"Large_Video_2col_4row",
"Medium_Video_1col_2row",
"Small_Video_1col_1row"
]
},
V1: {
classID: "V1",
snippetMin: 0,
aspectRatioMin: 0,
aspectRatioMax: 0,
isDefault: false,
type: "VideoCluster",
templateList: [
"Large_videoCluster",
"Medium_videoCluster",
"Small_videoCluster"
]
},
V2: {
classID: "V2",
snippetMin: 0,
aspectRatioMin: 0,
aspectRatioMax: 0,
isDefault: false,
type: "VideoCluster",
templateList: [
"Medium_videoCluster"
]
},
Z1: {
classID: "Z1",
snippetMin: 0,
aspectRatioMin: 0,
aspectRatioMax: 0,
isDefault: false,
type: "Zinio",
templateList: [
"Medium_Square_1col_4row"
]
},
Z2: {
classID: "Z2",
snippetMin: 0,
aspectRatioMin: 0,
aspectRatioMax: 0,
isDefault: false,
type: "Zinio",
templateList: [
"Large_Landscape_2col_4row",
"Medium_Landscape_1col_4row"
]
},
partnerA: {
classID: "partnerA",
snippetMin: 0,
aspectRatioMin: 0,
aspectRatioMax: 0,
isDefault: false,
type: "Sources",
templateList: [
"Small_Square_Source_1col_1row"
]
},
partnerB: {
classID: "partnerB",
snippetMin: 0,
aspectRatioMin: 0,
aspectRatioMax: 0,
isDefault: false,
type: "Sources",
templateList: [
"Small_Landscape_Source_1col_1row"
]
},
},
TemplateDefinitions:
{
Large_Square_2col_4row: {
classID: "Large_Square_2col_4row",
thumbnailHeight: 590,
thumbnailWidth: 610,
height: 59,
width: 61,
lastResort: false,
minHeadlineLength: 0,
maxHeadlineLength: 51
},
Large_Landscape_2col_3row: {
classID: "Large_Landscape_2col_3row",
thumbnailHeight: 406,
thumbnailWidth: 610,
height: 44,
width: 61,
lastResort: false,
minHeadlineLength: 0,
maxHeadlineLength: 51
},
Large_Landscape_2col_4row: {
classID: "Large_Landscape_2col_4row",
thumbnailHeight: 380,
thumbnailWidth: 570,
height: 59,
width: 61,
lastResort: false,
weight: 3,
minHeadlineLength: 0,
maxHeadlineLength: 77
},
Medium_Square_1col_4row: {
classID: "Medium_Square_1col_4row",
thumbnailHeight: 260,
thumbnailWidth: 260,
height: 59,
width: 30,
lastResort: false,
weight: 2,
minHeadlineLength: 0,
maxHeadlineLength: 77
},
Medium_Square_1col_3row: {
classID: "Medium_Square_1col_3row",
thumbnailHeight: 260,
thumbnailWidth: 260,
height: 44,
width: 30,
lastResort: false,
minHeadlineLength: 0,
maxHeadlineLength: 58
},
Medium_Landscape_1col_4row: {
classID: "Medium_Landscape_1col_4row",
thumbnailHeight: 174,
thumbnailWidth: 260,
height: 59,
width: 30,
lastResort: false,
weight: 2,
minHeadlineLength: 0,
maxHeadlineLength: 77
},
Medium_Landscape_1col_3row: {
classID: "Medium_Landscape_1col_3row",
thumbnailHeight: 174,
thumbnailWidth: 260,
height: 44,
width: 30,
lastResort: false,
minHeadlineLength: 0,
maxHeadlineLength: 77
},
Medium_Landscape_LongHeadline_1col_3row: {
classID: "Medium_Landscape_LongHeadline_1col_3row",
thumbnailHeight: 174,
thumbnailWidth: 260,
height: 44,
width: 30,
lastResort: false,
minHeadlineLength: 77
},
Medium_Portrait_1col_4row: {
classID: "Medium_Portrait_1col_4row",
thumbnailHeight: 390,
thumbnailWidth: 260,
height: 59,
width: 30,
lastResort: false,
weight: 2,
minHeadlineLength: 0,
maxHeadlineLength: 58
},
Small_Square_2col_2row: {
classID: "Small_Square_2col_2row",
thumbnailHeight: 100,
thumbnailWidth: 100,
height: 29,
width: 61,
lastResort: false
},
Small_Square_1col_2row: {
classID: "Small_Square_1col_2row",
thumbnailHeight: 100,
thumbnailWidth: 100,
height: 29,
width: 30,
lastResort: false,
minHeadlineLength: 0,
maxHeadlineLength: 58
},
None_1col_1row: {
classID: "None_1col_1row",
thumbnailHeight: 0,
thumbnailWidth: 0,
height: 14,
width: 30,
lastResort: true
},
None_1col_2row: {
classID: "None_1col_2row",
thumbnailHeight: 0,
thumbnailWidth: 0,
height: 29,
width: 30,
lastResort: false,
minHeadlineLength: 0,
maxHeadlineLength: 77
},
None_LongHeadline_1col_3row: {
classID: "None_LongHeadline_1col_3row",
thumbnailHeight: 0,
thumbnailWidth: 0,
height: 44,
width: 30,
lastResort: false,
minHeadlineLength: 77
},
None_1col_4row: {
classID: "None_1col_4row",
thumbnailHeight: 0,
thumbnailWidth: 0,
height: 59,
width: 30,
lastResort: false,
minHeadlineLength: 0,
maxHeadlineLength: 77
},
Large_Slideshow_2col_4row: {
classID: "Large_Slideshow_2col_4row",
thumbnailHeight: 380,
thumbnailWidth: 570,
height: 59,
width: 61,
lastResort: false,
weight: 2
},
Medium_Slideshow_1col_2row: {
classID: "Medium_Slideshow_1col_2row",
thumbnailHeight: 140,
thumbnailWidth: 260,
height: 29,
width: 30,
lastResort: false
},
Small_Slideshow_1col_1row: {
classID: "Small_Slideshow_1col_1row",
thumbnailHeight: 140,
thumbnailWidth: 300,
height: 14,
width: 30,
lastResort: true,
minHeadlineLength: 0,
maxHeadlineLength: 51
},
Large_Video_2col_4row: {
classID: "Large_Video_2col_4row",
thumbnailHeight: 380,
thumbnailWidth: 570,
height: 59,
width: 61,
lastResort: false,
weight: 2,
minHeadlineLength: 0,
maxHeadlineLength: 51
},
Medium_Video_1col_2row: {
classID: "Medium_Video_1col_2row",
thumbnailHeight: 140,
thumbnailWidth: 260,
height: 29,
width: 30,
lastResort: false
},
Small_Video_1col_1row: {
classID: "Small_Video_1col_1row",
thumbnailHeight: 140,
thumbnailWidth: 300,
height: 14,
width: 30,
lastResort: true
},
Large_videoCluster: {
classID: "Large_videoCluster",
thumbnailHeight: 440,
thumbnailWidth: 680,
height: 44,
width: 68,
lastResort: false,
weight: 2
},
Medium_videoCluster: {
classID: "Medium_videoCluster",
thumbnailHeight: 290,
thumbnailWidth: 450,
height: 29,
width: 45,
lastResort: false
},
Small_videoCluster: {
classID: "Small_videoCluster",
thumbnailHeight: 140,
thumbnailWidth: 220,
height: 14,
width: 22,
lastResort: true
},
Small_Square_Source_1col_1row: {
classID: "Small_Square_Source_1col_1row",
thumbnailHeight: 100,
thumbnailWidth: 100,
height: 14,
width: 13,
lastResort: false
},
Small_Landscape_Source_1col_1row: {
classID: "Small_Landscape_Source_1col_1row",
thumbnailHeight: 53,
thumbnailWidth: 156,
height: 7,
width: 18,
lastResort: false
},
},
RulesList:
[
{
type: "Frequency",
count: 1,
columns: 0,
templateList: [
"Large_Square_2col_4row"
]
},
{
type: "Frequency",
count: 1,
columns: 0,
templateList: [
"Large_Landscape_2col_3row"
]
},
{
type: "Frequency",
count: 1,
columns: 0,
templateList: [
"Large_Landscape_2col_4row"
]
},
{
type: "Frequency",
count: 2,
columns: 6,
templateList: [
"Large_Square_2col_4row",
"Large_Landscape_2col_3row",
"Large_Landscape_2col_4row",
"Large_Slideshow_2col_4row",
"Large_Video_2col_4row"
]
},
{
type: "Frequency",
count: 1,
columns: 4,
templateList: [
"Small_Square_2col_2row"
]
},
{
type: "Frequency",
count: 1,
columns: 4,
templateList: [
"Medium_Portrait_1col_4row"
]
},
{
type: "Frequency",
count: 1,
columns: 3,
templateList: [
"Medium_Portrait_1col_4row",
"Large_Square_2col_4row"
]
},
{
type: "Frequency",
count: 1,
columns: 4,
templateList: [
"Large_videoCluster"
]
},
{
type: "Position",
minX: 0,
minY: 1,
maxX: 100,
maxY: 100,
templateList: [
"Large_Square_2col_4row",
"Large_Landscape_2col_3row",
"Large_Landscape_2col_4row",
"Large_Slideshow_2col_4row",
"Large_Video_2col_4row"
]
}
]
});
})();
/* File: DefaultNewsConfig.js */
(function () {
"use strict";
WinJS.Namespace.define("CommonJS.News.EntityClusterConfig.DefaultNews",
{
TemplateClassMap:
{
B: {
classID: "B",
snippetMin: 650,
aspectRatioMin: 1.3,
aspectRatioMax: 1.9,
isDefault: false,
type: "News",
templateList: [
"Large_Landscape_2col_3row",
"Large_Landscape_2col_4row",
"Medium_Landscape_1col_4row",
"Medium_Landscape_1col_3row",
"Medium_Landscape_LongHeadline_1col_3row",
"None_1col_1row"
]
},
A: {
classID: "A",
snippetMin: 0,
aspectRatioMin: 1.3,
aspectRatioMax: 1.9,
isDefault: false,
type: "News",
templateList: [
"Large_Landscape_2col_3row",
"Medium_Landscape_1col_3row",
"Medium_Landscape_LongHeadline_1col_3row",
"None_1col_1row"
]
},
C: {
classID: "C",
snippetMin: 0,
aspectRatioMin: 0.5,
aspectRatioMax: 0.9,
isDefault: false,
type: "News",
templateList: [
"Medium_Portrait_1col_4row",
"None_LongHeadline_1col_3row",
"None_1col_2row",
"None_1col_1row"
]
},
I: {
classID: "I",
snippetMin: 650,
aspectRatioMin: 0.9,
aspectRatioMax: 1.3,
isDefault: false,
type: "News",
templateList: [
"Large_Square_2col_4row",
"Medium_Square_1col_4row",
"Medium_Square_1col_3row",
"Small_Square_1col_2row",
"None_LongHeadline_1col_3row",
"None_1col_1row"
]
},
E: {
classID: "E",
snippetMin: 0,
aspectRatioMin: 0.9,
aspectRatioMax: 1.3,
isDefault: false,
type: "News",
templateList: [
"Large_Square_2col_4row",
"Medium_Square_1col_4row",
"Medium_Square_1col_3row",
"Small_Square_1col_2row",
"None_LongHeadline_1col_3row",
"None_1col_1row"
]
},
F: {
classID: "F",
snippetMin: 0,
aspectRatioMin: 0,
aspectRatioMax: 0,
isDefault: false,
type: "News",
templateList: [
"None_LongHeadline_1col_3row",
"None_1col_2row",
"None_1col_1row"
]
},
H: {
classID: "H",
snippetMin: 650,
aspectRatioMin: 0,
aspectRatioMax: 0,
isDefault: true,
type: "News",
templateList: [
"None_LongHeadline_1col_3row",
"None_1col_2row",
"None_1col_1row",
"None_1col_4row"
]
},
G: {
classID: "G",
snippetMin: 0,
aspectRatioMin: 0,
aspectRatioMax: 0,
isDefault: true,
type: "News",
templateList: [
"None_LongHeadline_1col_3row",
"None_1col_2row",
"None_1col_1row"
]
},
S: {
classID: "S",
snippetMin: 0,
aspectRatioMin: 0,
aspectRatioMax: 0,
isDefault: false,
type: "Slideshow",
templateList: [
"Large_Slideshow_2col_4row",
"Medium_Slideshow_1col_2row",
"Small_Slideshow_1col_1row"
]
},
V: {
classID: "V",
snippetMin: 0,
aspectRatioMin: 0,
aspectRatioMax: 0,
isDefault: false,
type: "Video",
templateList: [
"Large_Video_2col_4row",
"Medium_Video_1col_2row",
"Small_Video_1col_1row"
]
},
V1: {
classID: "V1",
snippetMin: 0,
aspectRatioMin: 0,
aspectRatioMax: 0,
isDefault: false,
type: "VideoCluster",
templateList: [
"Large_videoCluster",
"Medium_videoCluster",
"Small_videoCluster"
]
},
V2: {
classID: "V2",
snippetMin: 0,
aspectRatioMin: 0,
aspectRatioMax: 0,
isDefault: false,
type: "VideoCluster",
templateList: [
"Medium_videoCluster"
]
},
partnerA: {
classID: "partnerA",
snippetMin: 0,
aspectRatioMin: 0,
aspectRatioMax: 0,
isDefault: false,
type: "Sources",
templateList: [
"Small_Square_Source_1col_1row"
]
},
partnerB: {
classID: "partnerB",
snippetMin: 0,
aspectRatioMin: 0,
aspectRatioMax: 0,
isDefault: false,
type: "Sources",
templateList: [
"Small_Landscape_Source_1col_1row"
]
},
AP_breaking: {
classID: "AP_breaking",
snippetMin: 0,
aspectRatioMin: 0,
aspectRatioMax: 0,
isDefault: false,
type: "AP",
templateList: [
"None_Breaking_AP"
]
},
AP_nonbreaking: {
classID: "AP_nonbreaking",
snippetMin: 0,
aspectRatioMin: 0,
aspectRatioMax: 0,
isDefault: false,
type: "AP",
templateList: [
"Medium_AP",
"None_AP"
]
},
S2: {
classID: "S2",
snippetMin: 0,
aspectRatioMin: 0,
aspectRatioMax: 0,
isDefault: false,
type: "Slideshow",
templateList: [
"Medium_slideshowCluster"
]
},
AP_bigstories: {
classID: "AP_bigstories",
snippetMin: 0,
aspectRatioMin: 0,
aspectRatioMax: 0,
isDefault: false,
type: "AP",
templateList: [
"Medium_AP_1col_1row"
]
},
P: {
classID: "P",
snippetMin: 0,
aspectRatioMin: 0,
aspectRatioMax: 0,
isDefault: false,
type: "PopularNow",
templateList: [
"Medium_Square_1col_3row"
]
},
},
TemplateDefinitions:
{
Medium_AP: {
classID: "Medium_AP",
thumbnailHeight: 160,
thumbnailWidth: 260,
height: 59,
width: 30,
lastResort: false,
minHeadlineLength: 0,
maxHeadlineLength: 0
},
None_AP: {
classID: "None_AP",
thumbnailHeight: 0,
thumbnailWidth: 0,
height: 59,
width: 30,
lastResort: true,
minHeadlineLength: 0,
maxHeadlineLength: 0
}, 
None_Breaking_AP: {
classID: "None_Breaking_AP",
thumbnailHeight: 0,
thumbnailWidth: 0,
height: 59,
width: 30,
lastResort: false,
minHeadlineLength: 0,
maxHeadlineLength: 0
},
Medium_AP_1col_1row: {
classID: "Medium_AP_1col_1row",
thumbnailHeight: 406,
thumbnailWidth: 610,
height: 44,
width: 61,
lastResort: false,
minHeadlineLength: 0,
maxHeadlineLength: 0
},
Large_Square_2col_4row: {
classID: "Large_Square_2col_4row",
thumbnailHeight: 590,
thumbnailWidth: 590,
height: 59,
width: 61,
lastResort: false,
minHeadlineLength: 0,
maxHeadlineLength: 51
},
Large_Landscape_2col_3row: {
classID: "Large_Landscape_2col_3row",
thumbnailHeight: 393,
thumbnailWidth: 590,
height: 44,
width: 61,
lastResort: false,
minHeadlineLength: 0,
maxHeadlineLength: 51
},
Large_Landscape_2col_4row: {
classID: "Large_Landscape_2col_4row",
thumbnailHeight: 393,
thumbnailWidth: 590,
height: 59,
width: 61,
lastResort: false,
weight: 3,
minHeadlineLength: 0,
maxHeadlineLength: 77
},
Medium_Square_1col_4row: {
classID: "Medium_Square_1col_4row",
thumbnailHeight: 280,
thumbnailWidth: 280,
height: 59,
width: 30,
lastResort: false,
weight: 2,
minHeadlineLength: 0,
maxHeadlineLength: 77
},
Medium_Square_1col_3row: {
classID: "Medium_Square_1col_3row",
thumbnailHeight: 280,
thumbnailWidth: 280,
height: 44,
width: 30,
lastResort: false,
minHeadlineLength: 0,
maxHeadlineLength: 58
},
Medium_Landscape_1col_4row: {
classID: "Medium_Landscape_1col_4row",
thumbnailHeight: 187,
thumbnailWidth: 280,
height: 59,
width: 30,
lastResort: false,
weight: 2,
minHeadlineLength: 0,
maxHeadlineLength: 77
},
Medium_Landscape_1col_3row: {
classID: "Medium_Landscape_1col_3row",
thumbnailHeight: 187,
thumbnailWidth: 280,
height: 44,
width: 30,
lastResort: false,
minHeadlineLength: 0,
maxHeadlineLength: 77
},
Medium_Landscape_LongHeadline_1col_3row: {
classID: "Medium_Landscape_LongHeadline_1col_3row",
thumbnailHeight: 187,
thumbnailWidth: 280,
height: 44,
width: 30,
lastResort: false,
minHeadlineLength: 77
},
Medium_Portrait_1col_4row: {
classID: "Medium_Portrait_1col_4row",
thumbnailHeight: 420,
thumbnailWidth: 280,
height: 59,
width: 30,
lastResort: false,
weight: 2,
minHeadlineLength: 0,
maxHeadlineLength: 58
},
Small_Square_2col_2row: {
classID: "Small_Square_2col_2row",
thumbnailHeight: 100,
thumbnailWidth: 100,
height: 29,
width: 61,
lastResort: false
},
Small_Square_1col_2row: {
classID: "Small_Square_1col_2row",
thumbnailHeight: 100,
thumbnailWidth: 100,
height: 29,
width: 30,
lastResort: false,
minHeadlineLength: 0,
maxHeadlineLength: 58
},
None_1col_1row: {
classID: "None_1col_1row",
thumbnailHeight: 0,
thumbnailWidth: 0,
height: 14,
width: 30,
lastResort: true
},
None_1col_2row: {
classID: "None_1col_2row",
thumbnailHeight: 0,
thumbnailWidth: 0,
height: 29,
width: 30,
lastResort: false,
minHeadlineLength: 0,
maxHeadlineLength: 77
},
None_LongHeadline_1col_3row: {
classID: "None_LongHeadline_1col_3row",
thumbnailHeight: 0,
thumbnailWidth: 0,
height: 44,
width: 30,
lastResort: false,
minHeadlineLength: 77
},
None_1col_4row: {
classID: "None_1col_4row",
thumbnailHeight: 0,
thumbnailWidth: 0,
height: 59,
width: 30,
lastResort: false,
minHeadlineLength: 0,
maxHeadlineLength: 77
},
Large_Slideshow_2col_4row: {
classID: "Large_Slideshow_2col_4row",
thumbnailHeight: 393,
thumbnailWidth: 590,
height: 59,
width: 61,
lastResort: false,
weight: 2,
minHeadlineLength: 0,
maxHeadlineLength: 51
},
Medium_Slideshow_1col_2row: {
classID: "Medium_Slideshow_1col_2row",
thumbnailHeight: 151,
thumbnailWidth: 280,
height: 29,
width: 30,
lastResort: false
},
Small_Slideshow_1col_1row: {
classID: "Small_Slideshow_1col_1row",
thumbnailHeight: 114,
thumbnailWidth: 280,
height: 14,
width: 30,
lastResort: true
},
Large_Video_2col_4row: {
classID: "Large_Video_2col_4row",
thumbnailHeight: 382,
thumbnailWidth: 590,
height: 59,
width: 61,
lastResort: false,
weight: 2,
minHeadlineLength: 0,
maxHeadlineLength: 51
},
Medium_Video_1col_2row: {
classID: "Medium_Video_1col_2row",
thumbnailHeight: 151,
thumbnailWidth: 280,
height: 29,
width: 30,
lastResort: false
},
Small_Video_1col_1row: {
classID: "Small_Video_1col_1row",
thumbnailHeight: 114,
thumbnailWidth: 280,
height: 14,
width: 30,
lastResort: true
},
Large_videoCluster: {
classID: "Large_videoCluster",
thumbnailHeight: 440,
thumbnailWidth: 680,
height: 44,
width: 68,
lastResort: false,
weight: 2
},
Medium_videoCluster: {
classID: "Medium_videoCluster",
thumbnailHeight: 290,
thumbnailWidth: 450,
height: 29,
width: 45,
lastResort: false
},
Small_videoCluster: {
classID: "Small_videoCluster",
thumbnailHeight: 140,
thumbnailWidth: 220,
height: 14,
width: 22,
lastResort: true
},
Medium_slideshowCluster: {
classID: "Medium_slideshowCluster",
thumbnailHeight: 290,
thumbnailWidth: 450,
height: 29,
width: 45,
lastResort: false
},
Small_Square_Source_1col_1row: {
classID: "Small_Square_Source_1col_1row",
thumbnailHeight: 100,
thumbnailWidth: 100,
height: 14,
width: 13,
lastResort: false
},
Small_Landscape_Source_1col_1row: {
classID: "Small_Landscape_Source_1col_1row",
thumbnailHeight: 53,
thumbnailWidth: 156,
height: 7,
width: 18,
lastResort: false
}
},
RulesList:
[
{
type: "Frequency",
count: 1,
columns: 0,
templateList: [
"Large_Square_2col_4row"
]
},
{
type: "Frequency",
count: 1,
columns: 0,
templateList: [
"Large_Landscape_2col_3row"
]
},
{
type: "Frequency",
count: 1,
columns: 0,
templateList: [
"Large_Landscape_2col_4row"
]
},
{
type: "Frequency",
count: 2,
columns: 6,
templateList: [
"Large_Square_2col_4row",
"Large_Landscape_2col_3row",
"Large_Landscape_2col_4row",
"Large_Slideshow_2col_4row",
"Large_Video_2col_4row"
]
},
{
type: "Frequency",
count: 1,
columns: 4,
templateList: [
"Small_Square_2col_2row"
]
},
{
type: "Frequency",
count: 1,
columns: 4,
templateList: [
"Medium_Portrait_1col_4row"
]
},
{
type: "Frequency",
count: 1,
columns: 3,
templateList: [
"Medium_Portrait_1col_4row",
"Large_Square_2col_4row"
]
},
{
type: "Frequency",
count: 1,
columns: 4,
templateList: [
"Large_videoCluster"
]
},
{
type: "Position",
minX: 0,
minY: 1,
maxX: 100,
maxY: 100,
templateList: [
"Large_Square_2col_4row",
"Large_Landscape_2col_3row",
"Large_Landscape_2col_4row",
"Large_Slideshow_2col_4row",
"Large_Video_2col_4row"
]
}
]
});
})();
/* File: ProcessListener.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function (WinJS) {
"use strict";
var News = WinJS.Namespace.define("CommonJS.News",
{
ProcessListener: WinJS.Class.mix(WinJS.Class.define(function () {
// ctor
this._populateSupplementalConfig();
},
{
_updateCustomCSS: function _updateCustomCSS() {
var config = CommonJS.News.SupplementalConfig;
if (config && config.CSS) {
var element = document.getElementById("EntityClusterSupplementalStyles");
if (!element) {
element = document.createElement("style");
element.id = "EntityClusterSupplementalStyles";
element.type = "text/css";
element.setAttribute("data-version", config.Version);
element.styleSheet.cssText = toStaticHTML(config.CSS);
document.body.appendChild(element);
}
else {
var versionAttribute = element.getAttribute("data-version");
if (versionAttribute !== config.Version) {
element.styleSheet.cssText = toStaticHTML(config.CSS);
element.setAttribute("data-version", config.Version);
}
}
}
},
_handleJSON: function _handleJSON(jsonString) {
if (jsonString) {
var json = JSON.parse(jsonString);
CommonJS.News.SupplementalConfig = json;
this._updateCustomCSS();
}
},
_populateSupplementalConfig: function _populateSupplementalConfig() {
var that = this;
var primaryConfigUrl = PlatformJS.Services.configuration.getDictionary("EntityClusterConfig").getString("SupplementalConfigURLPrimary");
if (primaryConfigUrl) {
this._fetchContent(primaryConfigUrl)
.then(
function (data) {
if (data && !data.isCachedResponse) {
that._handleJSON(data.dataString);
}
},
function (error) {
var secondaryConfigUrl = PlatformJS.Services.configuration.getDictionary("EntityClusterConfig").getString("SupplementalConfigURLSecondary");
if (secondaryConfigUrl) {
that._fetchContent(secondaryConfigUrl)
.then(
function (data) {
if (data && !data.isCachedResponse) {
that._handleJSON(data.dataString);
}
},
null,
function (data) {
if (data && data.cachedResponse) {
that._handleJSON(data.cachedResponse.dataString);
}
});
}
},
function (data) {
if (data && data.cachedResponse) {
that._handleJSON(data.cachedResponse.dataString);
}
});
}
},
_fetchContent: function _fetchContent(url) {
var dataSource = "CustomEntityClusterConfig";
var service = new PlatformJS.DataService.QueryService(dataSource);
var urlParams = PlatformJS.Collections.createStringDictionary();
if (url) {
urlParams["url"] = url;
}
var options = new Platform.DataServices.QueryServiceOptions();
options.bypassCache = false;
options.useCompactDataModel = true;
var that = this;
return service.downloadDataAsync(urlParams, null, null, options);
},
/// <summary>
/// Handles layout change event
/// </summary>
/// <param name="event">
/// Layout change event
/// </param>
/// <returns>
/// Nothing
/// </returns>
onLayoutChange: function onLayoutChange(event) { },
/// <summary>
/// Handles orientation change event
/// </summary>
/// <param name="event">
/// Orientation change event
/// </param>
/// <returns>
/// Nothing
/// </returns>
onOrientationChange: function onOrientationChange(event) { },
/// <summary>
/// Handles Suspend event
/// </summary>
/// <param name="event">
/// Suspend event
/// </param>
/// <returns>
/// Nothing
/// </returns>
onSuspending: function onSuspending(event) { },
/// <summary>
/// Handles Checkpoint event
/// </summary>
/// <param name="event">
/// Suspend suspend event
/// </param>
/// <returns>
/// Nothing
/// </returns>
onCheckpoint: function onCheckpoint(event) { },
/// <summary>
/// Handles Resume event
/// </summary>
/// <param name="event">
/// resume event
/// </param>
/// <returns>
/// Nothing
/// </returns>
onResuming: function onResuming(event) {
this._populateSupplementalConfig();
},
/// <summary>
/// Handles activation event
/// </summary>
/// <param name="event">
/// activation event
/// </param>
/// <returns>
/// Nothing
/// </returns>
onActivated: function onActivated(event) {
},
}, {
SupplementalConfig: null,
// end static type definition
}), WinJS.UI.DOMEventMixin)
});
})(WinJS);
/* File: EntityCluster.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function (WinJS) {
"use strict";
var NV = CommonJS.NavigableView;
var RENDER_COMPLETE = "rendercomplete";
var ERROR = "error";
var DEBUG = PlatformJS.isDebug;
//TODO: Eventually we need to remove the reference to the private members in the processManager (List of listeners). Also added if check temporarily to get it to work with sports but we'll end up changing the way files are referenced with the sports app.
if (PlatformJS.mainProcessManager) {
PlatformJS.mainProcessManager._processListeners.push(new CommonJS.News.ProcessListener());
}
var News = WinJS.Namespace.define("CommonJS.News",
{
/// <summary>
/// A Cluster with modules that represent news items. The items in the cluster conform
/// to AppEx.Common.NewsCluster.Response.NewsDescriptor. The module templates for the view assume this interface.
/// </summary>
EntityCluster: WinJS.Class.mix(WinJS.Class.define(function (domElement, options) {
// ctor
var that = this;
domElement.winControl = this;
this._domElement = domElement;
WinJS.Utilities.addClass(domElement, options.theme || "");
this._elements = [];
// TODO: move both predicate and completion handler as utility functions for common controls (northamerica\edesfos)
// [BEGIN]
// register itemclick event
Object.defineProperties(this, WinJS.Utilities.createEventProperties("itemclick"));
Object.defineProperties(this, WinJS.Utilities.createEventProperties("seemoreclick"));
PlatformJS.Utilities.registerItemClickProxy(domElement,
// predicate
function (element) {
return WinJS.Utilities.hasClass(element, "entityCluster");
},
// completion
function (element) {
var index = element.getAttribute("data-index");
//Template Selector Debugging Path
if (CommonJS.News.EntityCluster.TemplateSelectorDebuggingEnabled && DEBUG) {
that._handleDebugOnClick(index, element);
}
//Normal Launching of a news item through the onclick handler passed in by the app that instantiated the EC.
else {
//HACK: For the video control, will remove when the control is ready.
var disableclick = element.getAttribute("data-disableclick");
if (!disableclick) {
var isSeeMore = WinJS.Utilities.hasClass(element, "seeMore");
if (isSeeMore) {
that.dispatchEvent("seemoreclick", {});
}
else {
var itemWrappers = that._itemWrappers;
if (itemWrappers) {
var item = itemWrappers[index].item;
that.dispatchEvent("itemclick", {
item: item
});
}
}
}
}
});
// [END]
// register rendercomplete event
Object.defineProperties(this, WinJS.Utilities.createEventProperties(RENDER_COMPLETE));
// register error event
Object.defineProperties(this, WinJS.Utilities.createEventProperties(ERROR));
var unitWidth = this._unitWidth = options.unitWidth ? options.unitWidth : 31;
this._unitHeight = options.unitHeight ? options.unitHeight : 15;
this._configKey = options.configKey ? options.configKey : "EntityClusterDefaultNewsConfig";
//TODO: Get actual error message
this._noResultsMessage = options.noResultsMessage ? options.noResultsMessage : PlatformJS.Services.resourceLoader.getString("/platform/invalidEntry");
// set maxWidth on the view if maxColumnCount is provided in options
var maxColumnCount = this._maxColumnCount = options.maxColumnCount;
if (maxColumnCount) {
this._maxWidth = maxColumnCount * unitWidth - 1;
}
if (options.marginDistance && options.marginWidth) {
this._marginDistance = options.marginDistance * unitWidth - 1;
this._marginWidth = options.marginWidth;
if (maxColumnCount) {
var marginOccurances = Math.floor(maxColumnCount / options.marginDistance);
this._maxWidth += (marginOccurances * this._marginWidth);
}
}
this._adUnitId = options.adUnitId;
this._adsList = options.adsList;
var wrapper = this._containerWrapper = document.createElement("div");
wrapper.className = "entityClusterGridWrapper";
this._container = document.createElement("div");
this._container.className = "entityClusterGrid";
this._container.id = NV.createItemId();
this._adsWrapperControlList = [];
if (options.startingAd) {
var startingAdElt = this._startingAdElement = document.createElement("div");
var startingAd = this._startingAd = new PlatformJS.Ads.AdsWrapperControl(startingAdElt,
{
className: "adTypeThin",
adUnitId: this._adUnitId
});
WinJS.Utilities.addClass(startingAdElt, "startingAd");
}
// null check provider
if (options.mode === News.ClusterMode.dynamic && !options.provider) {
options.provider = new News.NewsProvider();
}
this._progressElt = null;
this._errorElt = null;
var cluster = PlatformJS.Utilities.getAncestorByClassName(domElement, "platformCluster");
if (cluster) {
var clusterLabelledBy = cluster.getAttribute("aria-labelledby");
if (clusterLabelledBy !== "") {
domElement.setAttribute("role", "listbox");
domElement.setAttribute("aria-labelledby", clusterLabelledBy);
}
}
this._templateClassMap = options.configObject.TemplateClassMap;
this._templateDefinitions = options.configObject.TemplateDefinitions;
this._rulesList = options.configObject.RulesList || [];
this._leadingItemIsLarge = false;
this._instrumentationEntryPoint = options.instrumentationEntryPoint;
// see more
var enableSeeMore = this._enableSeeMore = options.enableSeeMore;
if (enableSeeMore) {
if (PlatformJS.isDebug) {
if (maxColumnCount === undefined || maxColumnCount === null) {
throw "EntityCluster: When options.enableSeeMore is set to true, options.maxColumnCount must be provided as an integer.";
}
var marginDistance = options.marginDistance;
var marginWidth = options.marginWidth;
if (marginDistance || marginWidth) {
throw "EntityCluster: When options.enableSeeMore is set to true, options.marginDistance and options.marginWidth must not have a value.";
}
}
this._categoryName = options.categoryName;
}
// set options
WinJS.UI.setOptions(this, options);
if (this.retryHandler) {
this.addEventListener("retry", this.retryHandler);
}
},
{
// declare null aliases for auto-complete to work
_domElement: null,
_elements: null,
onitemclick: null,
onretry: null,
onrendercomplete: null,
onerror: null,
_progressElt: null,
_errorElt: null,
_newsItems: null,
_uniqueID: null,
_lastPromise: null,
_maxWidth: null,
_marginWidth: null,
_marginDistance: null,
_templateDefinitions: null,
_templateClassMap: null,
_configKey: null,
_noResultsElt: null,
_navigableViewOrchestrator: null,
_currentHeight: null,
_currentWidth: null,
_leadingItemIsLarge: null,
_instrumentationEntryPoint: null,
_debugTemplateSelectorData: null,
_supplementalConfigCacheKey: null,
_enableSeeMore: null,
_categoryName: null,
_maxColumnCount: null,
newsItems: {
get: function get() {
return this._newsItems;
}
},
onOrientationChange: function onOrientationChange(event) {
if (this._navigableViewOrchestrator) {
this._navigableViewOrchestrator.onOrientationChange(event);
}
},
onWindowResize: function onWindowResize(event) {
var dimensions = this._getDimensions();
var height = dimensions.height;
var width = dimensions.width;
if (height !== this._currentHeight || width !== this._currentWidth) {
this._handleRedraw();
}
},
onLayoutChange: function onLayoutChange(layoutState) {
var dimensions = this._getDimensions();
var height = dimensions.height;
var width = dimensions.width;
// when we go from a different state (snapped) to fullscreen, then we might have to redraw
if (layoutState === Windows.UI.ViewManagement.ApplicationViewState.fullScreenLandscape || layoutState === Windows.UI.ViewManagement.ApplicationViewState.filled) {
if (height !== this._currentHeight || width !== this._currentWidth) {
this._handleRedraw();
}
}
// portrait has a slightly different height to width comparison as opposed to landscape and filled
else if (layoutState === Windows.UI.ViewManagement.ApplicationViewState.fullScreenPortrait) {
if (width !== this._currentHeight || height !== this._currentWidth) {
this._handleRedraw();
}
}
},
_handleRedraw: function _handleRedraw() {
var that = this;
// normal case
if (that._response) {
that._cancelAndSwallow(that._lastPromise);
that._cancelAndSwallow(that._renderingPromise);
that._renderingPromise = that._renderResponse(that._response);
that._renderingPromise.done(function () {
if (that._navigableViewOrchestrator) {
that._navigableViewOrchestrator.onWindowResize(event);
}
});
}
// good state case
else if (that._itemWrappers) {
that._cancelAndSwallow(that._lastPromise);
that._cancelAndSwallow(that._renderingPromise);
that._renderingPromise = that._resizeLastKnownGoodState();
that._renderingPromise.done(function () {
if (that._navigableViewOrchestrator) {
that._navigableViewOrchestrator.onWindowResize(event);
}
});
}
},
_cancelAndSwallow: function _cancelAndSwallow(promise, e) {
if (promise) {
promise = promise
.then(null, function (err) {
if (err.message !== "Canceled") {
if (e) {
e(err);
} else {
throw err;
}
}
}, null);
promise.cancel();
}
},
dispose: function dispose() {
if (this.retryHandler) {
this.removeEventListener("retry", this.retryHandler);
}
this._clearAdControls();
if (this._startingAd && this._startingAd.dispose) {
this._startingAd.dispose();
}
if (this._navigableViewOrchestrator) {
this._navigableViewOrchestrator.dispose();
this._navigableViewOrchestrator = null;
}
},
/// <summary>
/// Invalidates the News cluster by querying the service and binding the resulting data to the view.
/// </summary>
render: function render() {
this._uniqueID = performance.now();
var that = this;
msWriteProfilerMark("CommonControls:EntityCluster:render:" + this._uniqueID + ":s");
return new WinJS.Promise(function (c, e, p) {
that._renderImpl(c, e, p);
});
},
_renderImpl: function _renderImpl(c, e, p) {
var that = this;
var mode = this.mode;
var cachedResponseRendered = false;
msWriteProfilerMark("CommonControls:EntityCluster:getNewsItems:s");
var promise = this._lastPromise = (mode === News.ClusterMode.dynamic) ? this._queryProviderAsync() : WinJS.Promise.wrap(this.dataSet || []);
promise
.then(
function (response) {
that._response = response;
if (response && !(response.isCachedResponse || response.serverResponseNotModified) || !cachedResponseRendered) {
var previousPromise = that._renderingPromise ? that._renderingPromise : WinJS.Promise.wrap(null);
that._renderingPromise = previousPromise
.then(
function () {
that._renderResponse(response)
.then(
function () {
c();
},
function (err) {
if (!PlatformJS.Utilities.isPromiseCanceled(err)) {
that._renderError(err);
}
c();
});
},
function (err) {
if (!PlatformJS.Utilities.isPromiseCanceled(err)) {
that._renderError(err);
}
c();
});
}
else {
that._renderingPromise = that._renderingPromise
.then(function () {
c();
});
}
},
function (err) {
var previousPromise = that._renderingPromise ? that._renderingPromise : WinJS.Promise.wrap(null);
that._renderingPromise = previousPromise.then(function () {
if (!PlatformJS.Utilities.isPromiseCanceled(err)) {
that._renderError(err);
}
c();
});
},
function (response) {
cachedResponseRendered = true;
that._response = response;
var previousPromise = that._renderingPromise ? that._renderingPromise : WinJS.Promise.wrap(null);
that._renderingPromise = previousPromise.then(function () {
return that._renderResponse(response);
});
});
},
_renderResponse: function _renderResponse(response) {
msWriteProfilerMark("CommonControls:EntityCluster:getNewsItems:e");
var that = this;
var domElement = that._domElement;
var newsItems = that._newsItems = (response ? response.newsItems : null);
if (!newsItems || newsItems.length === 0) {
this._showNoResults();
return WinJS.Promise.wrap(null);
}
if (!that.cacheID && response.dataCacheID && response.cacheID) {
that.cacheID = response.cacheID;
}
this._addSupplementalConfigs();
this._createDebugData(newsItems);
var itemWrappers = this._itemWrappers = that._createNewsItemsWrapper(newsItems);
// Extract Video info to support continuous video playback
var videoList = this._videoList = that._createVideoList(itemWrappers);
var key = response.clusterID ? [response.clusterID, "_", response.dataCacheID].join("") : response.dataCacheID;
key = response.uniqueResponseID ? [key, "_", response.uniqueResponseID].join("") : key;
return that._layoutGeneration(itemWrappers, response.dataCacheID, key)
.then(function (layout) {
msWriteProfilerMark("CommonControls:EntityCluster:generateLayoutNormal:e");
that._computeItemsToRender(layout, itemWrappers, videoList);
return that._renderClasses(layout);
});
},
_computeItemsToRender: function _computeItemsToRender(layout, newsItems, videoList) {
if (DEBUG) {
this._debugAppendLayoutInfo(layout);
}
var landscapeMax = this._computeNumberOfItems(layout.landscape);
var portraitMax = this._computeNumberOfItems(layout.portrait);
this._renderDOM(newsItems.slice(0, Math.max(landscapeMax, portraitMax) + 1), videoList);
},
_computeNumberOfItems: function _computeNumberOfItems(layout) {
var list = layout.itemList;
for (var i = list.length - 1; i > 0; i--) {
var item = list[i];
if (item) {
break;
}
}
return i;
},
_layoutGeneration: function _layoutGeneration(newsItems, dataCacheID, key) {
var that = this;
var templateValues = that._createTemplateSelectorOutput(newsItems);
msWriteProfilerMark("CommonControls:EntityCluster:generateLayoutNormal:s");
return that._getLayout(templateValues, dataCacheID, key);
},
_getLayout: function _getLayout(templateValues, dataCacheID, key) {
var that = this;
var response = this._response;
msWriteProfilerMark("CommonControls:EntityCluster:getLayout:s");
if (that.cacheID && dataCacheID && response.uniqueResponseID && !response.isLocalResponse) {
var layoutKey = this._generateLayoutCacheKey(key);
try {
return PlatformJS.Cache.CacheService.getInstance(that.cacheID).findEntry(layoutKey)
.then(function (cachedData) {
if (!cachedData) {
return that._createLayout(templateValues)
.then(function (json) {
var layout = that._layout = null;
if (json) {
layout = that._layout = JSON.parse(json);
}
msWriteProfilerMark("CommonControls:EntityCluster:getLayout:e");
if (layout) {
PlatformJS.Cache.CacheService.getInstance(that.cacheID).addEntry(layoutKey, layout);
} else {
throw "failed to parse json at EntityCluster";
}
return WinJS.Promise.wrap(layout);
});
}
else {
that._layout = cachedData.dataValue;
msWriteProfilerMark("CommonControls:EntityCluster:getLayout:e");
return WinJS.Promise.wrap(cachedData.dataValue);
}
});
}
catch (e) {
return that._createLayout(templateValues)
.then(function (json) {
var layout = that._layout = JSON.parse(json);
msWriteProfilerMark("CommonControls:EntityCluster:getLayout:e");
return WinJS.Promise.wrap(layout);
});
}
}
else {
return that._createLayout(templateValues)
.then(function (json) {
var layout = that._layout = JSON.parse(json);
msWriteProfilerMark("CommonControls:EntityCluster:getLayout:e");
return WinJS.Promise.wrap(layout);
});
}
},
_createLayout: function _createLayout(templateValues) {
var that = this;
var dimensions = this._getDimensions();
var height = this._currentHeight = dimensions.height;
var width = this._currentWidth = dimensions.width;
//HACK: This value is tied to the offset that is specified in imerssive.css regarding platformClusterContent and should be computed in the future. (tozha)
//[BEGIN]
var offset = this.heightOffset ? this.heightOffset : 160;
//[END]
var isCurrentLandscape;
var currentHeight = height - offset;
var otherOrientationScreenHeight = width - offset;
var value = Windows.UI.ViewManagement.ApplicationView.value;
if (value === Windows.UI.ViewManagement.ApplicationViewState.fullScreenPortrait) {
isCurrentLandscape = false;
}
else {
isCurrentLandscape = true;
}
var layoutOptions = {};
layoutOptions.leadingItemIsLarge = this._leadingItemIsLarge;
layoutOptions.screenHeight = currentHeight;
layoutOptions.configKey = that._configKey;
layoutOptions.unitWidth = that._unitWidth;
layoutOptions.unitHeight = that._unitHeight;
layoutOptions.rulesList = that._createRulesList(currentHeight);
var otherRulesList = that._createRulesList(otherOrientationScreenHeight);
layoutOptions.enableSeeMore = that._enableSeeMore;
if (that._maxWidth) {
layoutOptions.maxWidth = that._maxWidth;
}
if (that._marginDistance && that._marginWidth) {
layoutOptions.marginDistance = that._marginDistance;
layoutOptions.marginWidth = that._marginWidth;
if (that.subclusterAdFrequency) {
layoutOptions.subclusterAdFrequency = that.subclusterAdFrequency;
}
}
return CommonJS.News.OrientationLayoutGenerator.generateLayout(templateValues, layoutOptions, otherOrientationScreenHeight, otherRulesList, isCurrentLandscape);
},
_getLastColumnAndRow: function _getLastColumnAndRow(currentHeight) {
var unitHeight = this._unitHeight;
var rowCount = Math.floor(currentHeight / CommonJS.News.LayoutGenerator.RowHeight);
var lastRow = Math.floor((rowCount - (unitHeight - 1)) / unitHeight);
var lastColumn = this._maxColumnCount;
return {
column: lastColumn,
row: lastRow,
};
},
_createRulesList: function _createRulesList(currentHeight) {
var output = [];
// existing rules
var rulesList = this._rulesList;
if (rulesList) {
for (var i = 0, len = rulesList.length; i < len; i++) {
var rule = rulesList[i];
output.push(rule);
}
}
// handle see more
var enableSeeMore = this._enableSeeMore;
if (enableSeeMore) {
var last = this._getLastColumnAndRow(currentHeight);
var lastColumn = last.column;
var lastRow = last.row;
var minX = lastColumn - 1;
var maxX = lastColumn - 1;
var minY = lastRow;
var maxY = lastRow;
// this rule causes no templates to be rendered in the botttom right corner
var seeMoreRule = {
type: "Position",
minX: minX,
maxX: maxX,
minY: minY,
maxY: maxY,
isRestrictedArea: true,
templateList: [
"*",
]
};
output.push(seeMoreRule);
}
return output;
},
_renderDOM: function _renderDOM(newsItems, videoList) {
msWriteProfilerMark("CommonControls:EntityCluster:renderDom:s");
var that = this;
var container = this._container;
var wrapper = this._containerWrapper;
var domElement = this._domElement;
if (!wrapper.parentElement) {
domElement.appendChild(wrapper);
}
if (this._startingAdElement && !this._startingAdElement.parentElement) {
wrapper.appendChild(this._startingAdElement);
}
if (container.parentElement) {
wrapper.removeChild(container);
}
container.innerHTML = "";
for (var i = 0, leni = newsItems.length; i < leni; i++) {
var newsItem = newsItems[i];
newsItem.index = i;
var newsItemEl = this._createHTML(newsItems[i], videoList);
this._elements.push(newsItemEl);
container.appendChild(newsItemEl);
}
msWriteProfilerMark("CommonControls:EntityCluster:renderDom:e");
return WinJS.Promise.wrap(null);
},
isVideoClass: function isVideoClass(classType) {
return (classType === "Video" || classType === "VideoCluster");
},
isSlideshowClass: function isSlideshowClass(classType) {
return classType === "Slideshow";
},
isArticleClass: function isArticleClass(classType) {
return classType === "News";
},
_updateClasses: function _updateClasses(layout) {
// TODO: create a helper function for Landscape and Portrait and remove the duplicate code in this function
var itemListL = layout.landscape.itemList;
var itemListP = layout.portrait.itemList;
var container = this._container;
var children = container.childNodes;
var itemLengthL = itemListL.length;
var itemLengthP = itemListP.length;
var landscapeCSS = ["@media screen and (orientation: landscape) {"];
landscapeCSS.push(["#", container.id, " {width:", (layout.landscape.columnCount + 1) * 10, "px; height:", (layout.landscape.rowCount + 1) * 10, "px;} "].join(""));
var portraitCSS = ["@media screen and (orientation: portrait) {"];
portraitCSS.push(["#", container.id, " {width:", (layout.portrait.columnCount + 1) * 10, "px; height:", (layout.portrait.rowCount + 1) * 10, "px;} "].join(""));
for (var i = 0, leni = children.length; i < leni; i++) {
var cluster = children[i];
var className = [];
className.push("entityCluster");
if (i < itemLengthL) {
var id = NV.createItemId();
cluster.id = id;
var itemInfoL = itemListL[i];
if (itemInfoL) {
itemInfoL.id = id;
className.push(itemInfoL.classID + "_L");
var clusterLocationL = itemInfoL.clusterLocation.trim().split(" ");
for (var j = 0; j < clusterLocationL.length; j++) {
if (clusterLocationL[j] !== "") {
clusterLocationL[j] += "_L";
}
}
className.push(clusterLocationL.join(" "));
if (itemInfoL.thumbnailModifier !== "") {
className.push(itemInfoL.thumbnailModifier + "_L");
}
landscapeCSS.push(["#", id, " {-ms-grid-row:", itemInfoL.row, "; -ms-grid-column:", itemInfoL.column, ";} "].join(""));
}
else {
className.push("entityClusterHideLandscape");
}
}
else {
className.push("entityClusterHideLandscape");
}
if (i < itemLengthP) {
var itemInfoP = itemListP[i];
if (itemInfoP) {
itemInfoP.id = id;
className.push(itemInfoP.classID + "_P");
var clusterLocationP = itemInfoP.clusterLocation.trim().split(" ");
for (var k = 0; k < clusterLocationP.length; k++) {
if (clusterLocationP[k] !== "") {
clusterLocationP[k] += "_P";
}
}
className.push(clusterLocationP.join(" "));
if (itemInfoP.thumbnailModifier !== "") {
className.push(itemInfoP.thumbnailModifier + "_P");
}
portraitCSS.push(["#", id, " {-ms-grid-row:", itemInfoP.row, "; -ms-grid-column:", itemInfoP.column, ";} "].join(""));
}
else {
className.push("entityClusterHidePortrait");
}
}
else {
className.push("entityClusterHidePortrait");
}
cluster.className = className.join(" ");
}
landscapeCSS.push("}");
portraitCSS.push("}");
var adsStyleString = this._renderAds(layout);
var seeMoreStyleString = this._renderSeeMore(layout);
var styleString = [landscapeCSS.join(""), portraitCSS.join(""), adsStyleString, seeMoreStyleString];
var styleElement = this._styleElement = document.createElement("style");
styleElement.type = "text/css";
styleElement.styleSheet.cssText = styleString.join("");
container.appendChild(styleElement);
},
_renderClasses: function _renderClasses(layout) {
this._updateClasses(layout);
var container = this._container;
if (!container.parentElement) {
this._hideProgress();
this._hideError();
this._hideNoResults();
this._containerWrapper.appendChild(container);
}
if (this._navigableViewOrchestrator === null) {
this._createNavigableView();
}
this._telemetryInstrumentation();
msWriteProfilerMark("CommonControls:EntityCluster:render:" + this._uniqueID + ":e");
return WinJS.Promise.wrap(null);
},
_createNavigableView: function _createNavigableView() {
this._navigableViewOrchestrator = new CommonJS.NavigableView.NavigableViewOrchestrator(this._domElement, this._elements);
},
_clearAdControls: function _clearAdControls() {
var list = this._adsWrapperControlList;
for (var i = 0, leni = list.length; i < leni; i++) {
var ad = list[i];
if (ad && ad.dispose) {
ad.dispose();
}
}
this._adsWrapperControlList = [];
},
_renderAds: function _renderAds(layout) {
if (!this.subclusterAdFrequency) {
return "";
}
this._clearAdControls();
var container = this._container;
var adListP = layout.portrait.adList;
var adListL = layout.landscape.adList;
var adListLengthP = adListP.length;
var adListLengthL = adListL.length;
var adCount = Math.max(adListLengthP, adListLengthL);
var adListConfig = this._adsList;
var landscapeCSS = ["@media screen and (orientation: landscape) {"];
var portraitCSS = ["@media screen and (orientation: portrait) {"];
for (var i = 0, leni = adCount; i < leni; i++) {
var adElement = document.createElement("div");
var id = NV.createItemId();
var className = ["SubclusterAd"];
adElement.id = id;
var otherAdOptions = null;
if (adListConfig && adListConfig.length > 0) {
otherAdOptions = adListConfig[i % adListConfig.length];
}
var ad = new PlatformJS.Ads.AdsWrapperControl(adElement,
{
otherAdOptions: otherAdOptions,
className: "adTypeThin",
adUnitId: this._adUnitId
});
this._adsWrapperControlList.push(ad);
container.appendChild(adElement);
if (i < adListLengthL) {
var adInfoL = adListL[i];
if (adInfoL) {
landscapeCSS.push(["#", id, " {-ms-grid-row:", adInfoL.row, "; -ms-grid-column:", adInfoL.column, "; -ms-grid-column-span:", adInfoL.columnSpan, "; -ms-grid-row-span:", 60, ";} "].join(""));
}
else {
className.push("entityClusterHideLandscape");
}
}
else {
className.push("entityClusterHideLandscape");
}
if (i < adListLengthP) {
var adInfoP = adListP[i];
if (adInfoP) {
portraitCSS.push(["#", id, " {-ms-grid-row:", adInfoP.row, "; -ms-grid-column:", adInfoP.column, "; -ms-grid-column-span:", adInfoP.columnSpan, "; -ms-grid-row-span:", adInfoP.rowSpan, ";} "].join(""));
}
else {
className.push("entityClusterHidePortrait");
}
}
else {
className.push("entityClusterHidePortrait");
}
WinJS.Utilities.addClass(adElement, className.join(" "));
}
landscapeCSS.push("}");
portraitCSS.push("}");
return [landscapeCSS.join(""), portraitCSS.join("")].join("");
},
_renderSeeMore: function _renderSeeMore(layout) {
var style = "";
var enableSeeMore = this._enableSeeMore;
if (enableSeeMore) {
var container = this._container;
var seeMoreL = layout.landscape.seeMore;
var seeMoreP = layout.portrait.seeMore;
var landscapeCss = ["@media screen and (orientation: landscape) {"];
var portraitCss = ["@media screen and (orientation: portrait) {"];
var seeMoreElt = this._createSeeMoreHtml();
var id = seeMoreElt.id = NV.createItemId();
this._elements.push(seeMoreElt);
container.appendChild(seeMoreElt);
if (seeMoreL) {
landscapeCss.push(["#", id, " {-ms-grid-row:", seeMoreL.row, "; -ms-grid-column:", seeMoreL.column, "; -ms-grid-row-span:", seeMoreL.rowSpan, "; -ms-grid-column-span:", seeMoreL.columnSpan, ";}"].join(""));
}
else {
WinJS.Utilities.addClass(seeMoreElt, "entityClusterHideLandscape");
}
if (seeMoreP) {
portraitCss.push(["#", id, " {-ms-grid-row:", seeMoreP.row, "; -ms-grid-column:", seeMoreP.column, "; -ms-grid-row-span:", seeMoreP.rowSpan, "; -ms-grid-column-span:", seeMoreP.columnSpan, ";}"].join(""));
}
else {
WinJS.Utilities.addClass(seeMoreElt, "entityClusterHidePortrait");
}
landscapeCss.push("}");
portraitCss.push("}");
style = [landscapeCss.join(""), portraitCss.join("")].join("");
}
return style;
},
_createDebugData: function _createDebugData(newsItems) {
if (DEBUG) {
var landscapeDebug = [];
var portraitDebug = [];
for (var i = 0, leni = newsItems.length; i < leni; i++) {
landscapeDebug.push([""]);
portraitDebug.push([""]);
}
this._debugTemplateSelectorData = {
portrait: portraitDebug,
landscape: landscapeDebug
};
}
},
_createNewsItemsWrapper: function _createNewsItemsWrapper(newsItems) {
if (DEBUG) {
var landscapeDebug = [];
var portraitDebug = [];
}
var output = [];
for (var i = 0, leni = newsItems.length; i < leni; i++) {
var temp = {
item: newsItems[i],
templateClass: newsItems[i].templateClass,
index: i
};
output.push(temp);
if (DEBUG) {
landscapeDebug.push([""]);
portraitDebug.push([""]);
}
}
if (DEBUG) {
this._debugTemplateSelectorData = {
portrait: portraitDebug,
landscape: landscapeDebug
};
}
return output;
},
_createHTML: function _createHTML(newsItemWrapper, videoList) {
var templateClass = newsItemWrapper.templateClass;
var classMap = this._templateClassMap[templateClass];
if (classMap) {
var classType = classMap.type;
if (this.isSlideshowClass(classType)) {
return this._createSlideshowHtml(newsItemWrapper);
}
else if (this.isVideoClass(classType)) {
return this._createVideoHtml(newsItemWrapper, videoList);
}
else {
return this._createNormalHtml(newsItemWrapper);
}
}
else {
return this._createNormalHtml(newsItemWrapper);
}
},
_createVideoList: function _createVideoList(itemWrappers) {
//TODO: Move to onclick to find the entire list;
var videoList = [];
var index = 0;
for (var i = 0; i < itemWrappers.length; i++) {
// TODO: Use common function to et the template type
var itemWraper = itemWrappers[i];
var templateClass = itemWraper.templateClass;
var classType = templateClass && this._templateClassMap[templateClass] && this._templateClassMap[templateClass].type;
var item = itemWraper.item;
if (classType && this.isVideoClass(classType) && this._isValidVideo(item)) {
var optionSet = this._setVideoOptions(item.thumbnail.videoOptions);
optionSet.index = index++;
videoList.push(optionSet);
}
}
return videoList;
},
_isValidVideo: function _isValidVideo(item) {
return (item && item.thumbnail && item.thumbnail.videoOptions && item.thumbnail.videoOptions.videoSource);
},
_createSlideshowHtml: function _createSlideshowHtml(newsItemWrapper) {
var newsItem = newsItemWrapper.item;
var cluster = document.createElement("div");
cluster.setAttribute("data-index", newsItemWrapper.index);
var grid = document.createElement("div");
grid.className = "grid";
var thumbnail = document.createElement("div");
thumbnail.className = "thumbnail anchorTop";
var thumbnailData = newsItem.thumbnail;
if (thumbnailData) {
var imageSource;
var lowRes = newsItem.thumbnailLowRes;
if (lowRes) {
imageSource = {
lowResolutionUrl: lowRes.url,
highResolutionUrl: thumbnailData.url,
cacheId: "PlatformImageCache"
};
} else {
imageSource = {
url: thumbnailData.url,
cacheId: "PlatformImageCache"
};
}
var thumbnailImage = new CommonJS.ImageCard(thumbnail, {
alternateText: thumbnailData.altText || "",
classification: "small",
noIdentifier: 1,
imageSource: imageSource
});
}
grid.appendChild(thumbnail);
var gradient = document.createElement("div");
gradient.className = "gradient";
grid.appendChild(gradient);
var icon = document.createElement("div");
icon.className = "icon win-command";
var iconRing = document.createElement("div");
iconRing.className = "iconRing win-commandring win-commandicon";
var iconImage = document.createElement("div");
iconImage.className = "iconImage win-commandimage";
iconRing.appendChild(iconImage);
icon.appendChild(iconRing);
grid.appendChild(icon);
var headline = document.createElement("div");
headline.className = "headline";
headline.innerHTML = toStaticHTML(newsItem.title) || "";
var strippedTitle = headline.innerText;
var dir = PlatformJS.Utilities.getTextDirection(strippedTitle);
headline.setAttribute("dir", dir);
grid.appendChild(headline);
var kicker = document.createElement("div");
WinJS.Utilities.addClass(kicker, "kicker");
kicker.innerText = newsItem.kicker || "";
kicker.setAttribute("dir", dir);
grid.appendChild(kicker);
var source = document.createElement("div");
source.className = "source";
source.innerText = newsItem.source || "";
grid.appendChild(source);
var publishTime = document.createElement("div");
var publishClassName = "publishTime";
if (!newsItem.source) {
publishClassName += " noSource";
}
publishTime.className = publishClassName;
publishTime.innerText = this._getEffectiveTime(newsItem);
grid.appendChild(publishTime);
var sourceImage = document.createElement("div");
sourceImage.className = "sourceImage fitBoth";
var sourceImageUrl = newsItem.sourceImageUrl;
if (sourceImageUrl) {
var sourceImageImage = new CommonJS.ImageCard(sourceImage, {
alternateText: newsItem.source || "",
classification: "tiny",
noIdentifier: 1,
imageSource: {
url: sourceImageUrl,
cacheId: "PlatformImageCache"
}
});
}
else {
sourceImage.style.display = "none";
}
grid.appendChild(sourceImage);
var snippet = document.createElement("div");
snippet.className = "snippet";
snippet.innerHTML = toStaticHTML(newsItem.snippet) || "";
snippet.setAttribute("dir", dir);
if (sourceImageUrl || this._getEffectiveTime(newsItem) || newsItem.source) {
snippet.style.display = "none";
}
grid.appendChild(snippet);
var miniLine = document.createElement("div");
miniLine.className = "miniLine";
grid.appendChild(miniLine);
cluster.appendChild(grid);
PlatformJS.Utilities.enablePointerUpDownAnimations(cluster);
cluster.setAttribute("role", "option");
cluster.setAttribute("tabindex", 0);
return cluster;
},
_createVideoHtml: function _createVideoHtml(newsItemWrapper, videoList) {
var newsItem = newsItemWrapper.item;
var cluster = document.createElement("div");
cluster.setAttribute("data-index", newsItemWrapper.index);
var grid = document.createElement("div");
grid.className = "grid";
var thumbnail = document.createElement("div");
thumbnail.className = "thumbnail anchorTop";
var thumbnailData = newsItem.thumbnail;
if (thumbnailData) {
var imageSource;
var lowRes = newsItem.thumbnailLowRes;
if (lowRes) {
imageSource = {
lowResolutionUrl: lowRes.url,
highResolutionUrl: thumbnailData.url,
cacheId: "PlatformImageCache"
};
} else {
imageSource = {
url: thumbnailData.url,
cacheId: "PlatformImageCache"
};
}
var thumbnailImage = new CommonJS.ImageCard(thumbnail, {
alternateText: thumbnailData.altText || "",
classification: "small",
noIdentifier: 1,
imageSource: imageSource
});
}
grid.appendChild(thumbnail);
var gradient = document.createElement("div");
gradient.className = "gradient";
grid.appendChild(gradient);
var icon = document.createElement("div");
icon.className = "icon win-command";
var iconRing = document.createElement("div");
iconRing.className = "iconRing win-commandring win-commandicon";
var iconImage = document.createElement("div");
iconImage.className = "iconImage win-commandimage";
iconRing.appendChild(iconImage);
icon.appendChild(iconRing);
grid.appendChild(icon);
var headline = document.createElement("div");
headline.className = "headline";
headline.innerHTML = toStaticHTML(newsItem.title) || "";
var strippedTitle = headline.innerText;
var dir = PlatformJS.Utilities.getTextDirection(strippedTitle);
headline.setAttribute("dir", dir);
grid.appendChild(headline);
var kicker = document.createElement("div");
kicker.className = "kicker";
kicker.innerText = newsItem.kicker || "";
kicker.setAttribute("dir", dir);
grid.appendChild(kicker);
var source = document.createElement("div");
source.className = "source";
source.innerText = newsItem.source || "";
grid.appendChild(source);
var publishTime = document.createElement("div");
var publishClassName = "publishTime";
if (!newsItem.source) {
publishClassName += " noSource";
}
publishTime.className = publishClassName;
publishTime.innerText = this._getEffectiveTime(newsItem);
grid.appendChild(publishTime);
var sourceImage = document.createElement("div");
sourceImage.className = "sourceImage fitBoth";
var sourceImageUrl = newsItem.sourceImageUrl;
if (sourceImageUrl) {
var sourceImageImage = new CommonJS.ImageCard(sourceImage, {
alternateText: newsItem.source || "",
classification: "tiny",
noIdentifier: 1,
imageSource: {
url: sourceImageUrl,
cacheId: "PlatformImageCache"
}
});
}
else {
sourceImage.style.display = "none";
}
grid.appendChild(sourceImage);
var snippet = document.createElement("div");
snippet.className = "snippet";
snippet.innerHTML = toStaticHTML(newsItem.snippet) || "";
snippet.setAttribute("dir", dir);
if (sourceImageUrl || this._getEffectiveTime(newsItem) || newsItem.source) {
snippet.style.display = "none";
}
grid.appendChild(snippet);
var miniLine = document.createElement("div");
miniLine.className = "miniLine";
grid.appendChild(miniLine);
var wrapperInnerDiv = document.createElement("div");
wrapperInnerDiv.style.width = "100%";
wrapperInnerDiv.style.height = "100%";
wrapperInnerDiv.appendChild(grid);
var wrapper = new CommonJS.VideoWrapper(cluster, { subElement: wrapperInnerDiv, videoOptions: thumbnailData.videoOptions, videoList: videoList });
cluster.setAttribute("role", "option");
cluster.setAttribute("tabindex", 0);
PlatformJS.Utilities.enablePointerUpDownAnimations(cluster);
//HACK: For the video control, will remove when the control is ready.
cluster.setAttribute("data-disableclick", true);
return cluster;
},
_createNormalHtml: function _createNormalHtml(newsItemWrapper) {
var newsItem = newsItemWrapper.item;
var cluster = document.createElement("div");
cluster.setAttribute("data-index", newsItemWrapper.index);
var grid = document.createElement("div");
grid.className = "grid";
var thumbnail = document.createElement("div");
thumbnail.className = "thumbnail anchorTop";
var thumbnailData = newsItem.thumbnail;
if (thumbnailData) {
var imageSource;
var lowRes = newsItem.thumbnailLowRes;
if (lowRes) {
imageSource = {
lowResolutionUrl: lowRes.url,
highResolutionUrl: thumbnailData.url,
cacheId: "PlatformImageCache"
};
} else {
imageSource = {
url: thumbnailData.url,
cacheId: "PlatformImageCache"
};
}
var thumbnailImage = new CommonJS.ImageCard(thumbnail, {
alternateText: thumbnailData.altText || "",
classification: "small",
noIdentifier: 1,
imageSource: imageSource
});
}
grid.appendChild(thumbnail);
var gradient = document.createElement("div");
gradient.className = "gradient";
grid.appendChild(gradient);
var headline = document.createElement("div");
headline.className = "headline";
var strippedTitle;
if (newsItem.isLocked) {
var headlineSpan = document.createElement("span");
if (this._containsHTML(newsItem.title)) {
headlineSpan.className = "payWallLockedHTML";
}
else {
headlineSpan.className = "payWallLockedNormal";
}
headlineSpan.innerHTML = toStaticHTML(newsItem.title) || "";
strippedTitle = headlineSpan.innerText;
headline.appendChild(headlineSpan);
}
else {
headline.innerHTML = toStaticHTML(newsItem.title) || "";
strippedTitle = headline.innerText;
}
grid.appendChild(headline);
var dir = PlatformJS.Utilities.getTextDirection(strippedTitle);
headline.setAttribute("dir", dir);
var imageAttribution = document.createElement("div");
imageAttribution.className = "imageAttribution";
imageAttribution.innerText = newsItem.imageAttribution || "";
imageAttribution.setAttribute("dir", dir);
grid.appendChild(imageAttribution);
var kicker = document.createElement("div");
kicker.className = "kicker";
kicker.innerText = newsItem.kicker || "";
kicker.setAttribute("dir", dir);
grid.appendChild(kicker);
var sourceImage = document.createElement("div");
sourceImage.className = "sourceImage fitBoth";
var sourceImageUrl = newsItem.sourceImageUrl;
if (sourceImageUrl) {
var sourceImageImage = new CommonJS.ImageCard(sourceImage, {
alternateText: newsItem.source || "",
classification: "tiny",
noIdentifier: 1,
imageSource: {
url: sourceImageUrl,
cacheId: "PlatformImageCache"
}
});
}
else {
sourceImage.style.display = "none";
}
grid.appendChild(sourceImage);
var source = document.createElement("div");
source.className = "source";
if (!sourceImageUrl) {
WinJS.Utilities.addClass(source, "noSourceImage");
}
source.innerText = newsItem.source || "";
grid.appendChild(source);
var publishTime = document.createElement("div");
publishTime.className = "publishTime";
if (!sourceImageUrl) {
WinJS.Utilities.addClass(publishTime, "noSourceImage");
}
publishTime.innerText = this._getEffectiveTime(newsItem);
grid.appendChild(publishTime);
var snippet = document.createElement("div");
snippet.className = "snippet";
snippet.innerHTML = toStaticHTML(newsItem.snippet) || "";
snippet.setAttribute("dir", dir);
grid.appendChild(snippet);
cluster.appendChild(grid);
cluster.setAttribute("role", "option");
cluster.setAttribute("tabindex", 0);
PlatformJS.Utilities.enablePointerUpDownAnimations(cluster);
return cluster;
},
_createSeeMoreHtml: function _createSeeMoreHtml() {
var elt = document.createElement("div");
WinJS.Utilities.addClass(elt, "entityCluster seeMore");
elt.setAttribute("role", "option");
elt.setAttribute("tabindex", 0);
PlatformJS.Utilities.enablePointerUpDownAnimations(elt);
var gridElt = document.createElement("div");
WinJS.Utilities.addClass(gridElt, "grid");
elt.appendChild(gridElt);
var borderElt = document.createElement("div");
WinJS.Utilities.addClass(borderElt, "border");
gridElt.appendChild(borderElt);
var moreElt = document.createElement("div");
WinJS.Utilities.addClass(moreElt, "more");
moreElt.innerText = CommonJS.resourceLoader.getString("/platform/More");
gridElt.appendChild(moreElt);
var categoryNameElt = document.createElement("div");
WinJS.Utilities.addClass(categoryNameElt, "categoryName");
categoryNameElt.innerText = this._categoryName;
gridElt.appendChild(categoryNameElt);
var chevronElt = document.createElement("div");
WinJS.Utilities.addClass(chevronElt, "chevron");
gridElt.appendChild(chevronElt);
return elt;
},
_getEffectiveTime: function _getEffectiveTime(newsItem) {
return newsItem.updatedTime || newsItem.publishTime || "";
},
_stripHTML: function _stripHTML(string) {
if (string) {
return string.replace(/<(?:.|\n)*?>/gm, "");
}
else {
return "";
}
},
_containsHTML: function _containsHTML(string) {
if (string) {
var strippedString = this._stripHTML(string);
if (string === strippedString) {
return false;
}
else {
return true;
}
}
else {
return false;
}
},
_createTemplateSelectorOutput: function _createTemplateSelectorOutput(items) {
var that = this;
var templateSelectorOutput = [];
for (var i = 0, leni = items.length; i < leni; i++) {
var item = items[i];
templateSelectorOutput.push(this._templateSelector(item, i));
}
return templateSelectorOutput;
},
_templateSelector: function _templateSelector(item, i) {
////Test Purposes to allow us to pick a specific template
//if (item.item.template) {
//    var outputTemplates = [];
//    outputTemplates.push(this._createTemplateInfo(item.item.template));
//    return outputTemplates;
//}
var itemTemplateClass = item.templateClass;
var templateClass = this._templateClassMap[itemTemplateClass];
if (DEBUG) {
this._writeToDebugData("The template class that was provided by the data was: " + itemTemplateClass + ".\n", i, true, true);
}
if (itemTemplateClass && !templateClass) {
if (DEBUG) {
this._writeToDebugData("This template class was not defined in the config so it will be dropped.\n", i, true, true);
}
return this._getTemplatesForItem(item);
}
if (templateClass) {
this._sanitizeTemplateClass(item);
}
else {
this._generateTemplateClass(item);
}
this._checkLeadingVideoOrSlideshow(item, i);
return this._getTemplatesForItem(item);
},
_checkLeadingVideoOrSlideshow: function _checkLeadingVideoOrSlideshow(item, i) {
if (i === 0) {
var itemTemplateClass = item.templateClass;
var templateClass = this._templateClassMap[itemTemplateClass];
var classType = templateClass ? templateClass.type : null;
if (this.isVideoClass(classType) || this.isSlideshowClass(classType)) {
this._leadingItemIsLarge = true;
}
}
},
_validateVideoOptions: function _validateVideoOptions(itemWrapper, classType) {
if (this.isVideoClass(classType)) {
if (!itemWrapper.item.thumbnail) {
itemWrapper.item.thumbnail = {
width: 0,
height: 0,
url: "",
name: ""
};
}
var thumbnailData = itemWrapper.item.thumbnail;
if (!thumbnailData.videoOptions) {
thumbnailData.videoOptions = {};
thumbnailData.videoOptions.videoSource = "";
}
}
},
_setVideoOptions: function _setVideoOptions(videoOptions) {
videoOptions.fullscreen = "true";
videoOptions.adUnitId = this.videoAdUnitId;
videoOptions.enableVideoAds = this.enableVideoAds;
videoOptions.instrumentationEntryPoint = this._instrumentationEntryPoint;
return videoOptions;
},
_sanitizeTemplateClass: function _sanitizeTemplateClass(itemWrapper) {
var item = itemWrapper.item;
var index = itemWrapper.index;
var itemTemplateClass = itemWrapper.templateClass;
var valid = false;
var thumbnail = item.thumbnail;
var classMap = this._templateClassMap;
var snippetLength = item.snippet ? item.snippet.length : 0;
var tempClass = classMap[itemTemplateClass];
if (tempClass) {
this._validateVideoOptions(itemWrapper, tempClass.type);
var snippetMin = tempClass["snippetMin"];
var validSnippet = (snippetMin && snippetLength >= snippetMin) || (!snippetMin);
if (thumbnail && thumbnail.height > 0 && thumbnail.width > 0) {
var aspectRatio = thumbnail.width / thumbnail.height;
var aspectRatioMin = tempClass["aspectRatioMin"];
var aspectRatioMax = tempClass["aspectRatioMax"];
if (validSnippet) {
if (aspectRatioMin && aspectRatioMax) {
if (aspectRatio < aspectRatioMax && aspectRatio >= aspectRatioMin) {
valid = true;
}
else {
if (DEBUG) {
this._writeToDebugData("This item didn't fall within the desired aspectRatio range. It should fall between: " + aspectRatioMin + " and " + aspectRatioMax + ", but its actual aspect ratio is: " + aspectRatio + ".\n", index, true, true);
}
}
}
else {
valid = true;
}
}
else {
if (DEBUG) {
this._writeToDebugData("This item didn't satisfy the snippet minimum. This was required by the template class: " + snippetMin + " and " + snippetLength + " was provided in the data.\n", index, true, true);
}
}
}
else {
if (validSnippet) {
valid = true;
}
else {
if (DEBUG) {
this._writeToDebugData("This item didn't fall within the desired snippet range for the given class.\n", index, true, true);
}
}
}
}
if (!valid) {
this._generateTemplateClass(itemWrapper);
}
},
_generateTemplateClass: function _generateTemplateClass(itemWrapper) {
var item = itemWrapper.item;
var index = itemWrapper.index;
var thumbnail = item.thumbnail;
var snippetLength = item.snippet ? item.snippet.length : 0;
var classMap = this._templateClassMap;
for (var key in classMap) {
var current = classMap[key];
if (current["type"] !== "News") {
continue;
}
var aspectRatioMin = current["aspectRatioMin"];
var aspectRatioMax = current["aspectRatioMax"];
var snippetMin = current["snippetMin"];
if (thumbnail && thumbnail.width > 0 && thumbnail.height > 0) {
if (aspectRatioMin && aspectRatioMax) {
var aspectRatio = thumbnail.width / thumbnail.height;
if (aspectRatio >= aspectRatioMin && aspectRatio < aspectRatioMax) {
if (snippetMin) {
if (snippetLength >= snippetMin) {
itemWrapper.templateClass = current["classID"];
if (DEBUG) {
this._writeToDebugData("This template class that was generated for this item is: " + current["classID"] + ".\n", index, true, true);
}
return;
}
}
else {
itemWrapper.templateClass = current["classID"];
if (DEBUG) {
this._writeToDebugData("This template class that was generated for this item is: " + current["classID"] + ".\n", index, true, true);
}
return;
}
}
}
else {
if (snippetMin) {
if (snippetLength >= snippetMin) {
itemWrapper.templateClass = current["classID"];
if (DEBUG) {
this._writeToDebugData("This template class that was generated for this item is: " + current["classID"] + ".\n", index, true, true);
}
return;
}
}
else {
itemWrapper.templateClass = current["classID"];
if (DEBUG) {
this._writeToDebugData("This template class that was generated for this item is: " + current["classID"] + ".\n", index, true, true);
}
return;
}
}
}
else {
if (current["isDefault"]) {
if ((snippetMin && snippetLength >= snippetMin) || (!snippetMin)) {
itemWrapper.templateClass = current["classID"];
if (DEBUG) {
this._writeToDebugData("This template class that was generated for this item is: " + current["classID"] + ".\n", index, true, true);
}
return;
}
}
}
}
},
_validateThumbnailSizes: function _validateThumbnailSizes(item, template, outputTemplates, index) {
var templateDim = this._templateDefinitions[template];
var thumbnailHeight = templateDim["thumbnailHeight"];
var thumbnailWidth = templateDim["thumbnailWidth"];
var modifier = "";
var thumbnail = item.thumbnail;
if (thumbnailHeight && thumbnailWidth) {
if (thumbnail) {
var actualThumbnailHeight = thumbnail.height;
var actualThumbnailWidth = thumbnail.width;
if (actualThumbnailHeight < thumbnailHeight || actualThumbnailWidth < thumbnailWidth) {
if (DEBUG) {
this._writeToDebugData("This template: " + template + " was thrown out because it required a thumbnail size of " + thumbnailWidth + " by " + thumbnailHeight + ", but the actual data has a size of " + actualThumbnailWidth + " by " + actualThumbnailHeight + ".\n", index, true, true);
}
return;
}
var aspectRatio = thumbnailWidth / thumbnailHeight;
var actualAspectRatio = actualThumbnailWidth / actualThumbnailHeight;
if (actualAspectRatio < aspectRatio) {
modifier = "fitWidth";
}
else {
modifier = "fitHeight";
}
}
else {
if (DEBUG) {
this._writeToDebugData("This template: " + template + " was thrown out because it required a thumbnail and the data didn't have one.\n", index, true, true);
}
return;
}
}
var templateInfo = {
classID: templateDim.classID,
height: templateDim.height,
width: templateDim.width,
thumbnailModifier: modifier,
lastResort: templateDim.lastResort,
weight: templateDim.weight
};
this._validateHeadlineLengths(item, templateInfo, templateDim, outputTemplates, index);
},
_validateHeadlineLengths: function _validateHeadlineLengths(item, templateInfo, templateDim, outputTemplates, index) {
var minHeadlineLength = templateDim["minHeadlineLength"] || 0;
var maxHeadlineLength = templateDim["maxHeadlineLength"] || Number.MAX_VALUE;
var strippedHeadline = this._stripHTML(item.title);
var length = strippedHeadline.length;
if (length >= minHeadlineLength && length <= maxHeadlineLength) {
outputTemplates.push(templateInfo);
}
else {
if (DEBUG) {
this._writeToDebugData("This template: " + templateInfo.classID + " was thrown out because it required a headline range of: " + minHeadlineLength + " to " + maxHeadlineLength + ", but the actual data had a length of " + length + ".\n", index, true, true);
}
}
},
_getTemplatesForItem: function _getTemplatesForItem(itemWrapper) {
var outputTemplates = [];
var itemTemplateClass = itemWrapper.templateClass;
var classMap = this._templateClassMap[itemTemplateClass];
if (!itemTemplateClass || !classMap) {
return null;
}
var item = itemWrapper.item;
var templateList = classMap["templateList"];
for (var i = 0, ilen = templateList.length; i < ilen; i++) {
this._validateThumbnailSizes(item, templateList[i], outputTemplates, itemWrapper.index);
}
return outputTemplates;
},
_createTemplateInfo: function _createTemplateInfo(template) {
//Helper method for testing that lets us get individual template
var templateDims = this._templateDefinitions;
var templateDim = templateDims[template];
var height = templateDim["height"];
var width = templateDim["width"];
var templateInfo = {
classID: template,
height: height,
width: width,
thumbnailModifier: "fitBoth",
lastResort: false,
};
return templateInfo;
},
_generateLayoutCacheKey: function _generateLayoutCacheKey(datakey) {
var dimensions = this._getDimensions();
var height = this._currentHeight = dimensions.height;
var width = this._currentWidth = dimensions.width;
var supplementalKey = this._supplementalConfigCacheKey;
var key = ["EntityClusterLayout_", datakey, "_"];
if (height > width) {
key.push(height, "_", width);
}
else {
key.push(width, "_", height);
}
if (supplementalKey) {
key.push("_", supplementalKey);
}
return key.join("");
},
_getDimensions: function _getDimensions() {
var value = Windows.UI.ViewManagement.ApplicationView.value;
var height = window.outerHeight;
//HACK: This is so we get the correct height when going from fill'ed mode to portrait mode. We're adding the static 342 pixels in order to offset for the space lost in the snapped channel. (tozha)
//[BEGIN]
var width = value === Windows.UI.ViewManagement.ApplicationViewState.filled ? window.outerWidth + 342 : window.outerWidth;
//[END]
return { width: width, height: height };
},
_telemetryInstrumentation: function _telemetryInstrumentation() {
if (this.telemetry && this.telemetry.getCurrentImpression) {
var impression = this.telemetry.getCurrentImpression();
if (impression) {
var layout = this._layout;
var value = Windows.UI.ViewManagement.ApplicationView.value;
var currentLayout;
if (value === Windows.UI.ViewManagement.ApplicationViewState.fullScreenPortrait) {
currentLayout = layout.portrait;
}
else {
currentLayout = layout.landscape;
}
var orderedRenderList = currentLayout.orderedRenderList;
//Add content to the impression
this._addTelemetryContent(impression, orderedRenderList);
//Log content 
impression.logContent(Platform.Telemetry.LogLevel.normal);
//Add content view to the impression
this._addTelemetryContentView(impression, orderedRenderList);
}
}
},
_addTelemetryContent: function _addTelemetryContent(impression, orderedRenderList) {
if (orderedRenderList) {
for (var i = 0, leni = orderedRenderList.length; i < leni; i++) {
var item = orderedRenderList[i];
var telemetryInfo;
if (item.isAd) {
telemetryInfo = this._getAdsTelemetryInfo(item);
}
else {
telemetryInfo = this._getTileTelemetryInfo(item);
}
if (telemetryInfo) {
var k = impression.addContent(
telemetryInfo.sourceName,
telemetryInfo.partnerCode,
telemetryInfo.contentId,
telemetryInfo.type,
telemetryInfo.date,
telemetryInfo.uri,
telemetryInfo.slug,
telemetryInfo.isSummary,
telemetryInfo.worth,
telemetryInfo.isAd,
telemetryInfo.adCampaign);
//Used to log content view
item.telemetry = {
k: k
};
//Save the k index in the itemwrappers for the onclick function
if (!item.isAd) {
var itemWrapper = this._itemWrappers[item.realIndex];
itemWrapper.item.telemetry = {
k: k
};
}
}
}
}
},
_addTelemetryContentView: function _addTelemetryContentView(impression, orderedRenderList) {
if (orderedRenderList) {
var level = Platform.Telemetry.LogLevel.normal;
for (var i = 0, leni = orderedRenderList.length; i < leni; i++) {
var item = orderedRenderList[i];
var telemetry = item.telemetry;
if (telemetry && typeof telemetry.k === "number") {
impression.logContentView(
level,          //Level
telemetry.k,    //K index
"unknown",      //View Mechanism is unknown
true,           //Log only if not viewed
null            //Progress
);
}
}
}
},
_getTileTelemetryInfo: function _getTileTelemetryInfo(item) {
var itemWrapper = this._itemWrappers[item.realIndex];
var newsItem = itemWrapper.item;
var itemTemplateClass = itemWrapper.templateClass;
var templateClass = this._templateClassMap[itemTemplateClass];
var type;
if (templateClass) {
type = this._convertTypeToTelemetryType(templateClass.type);
}
var date;
if (newsItem.published) {
date = this._convertUtcTime(newsItem.published.utctime);
}
else {
date = new Date(1, 1, 1, 0, 0, 0, 0);
}
var slug = this._stripHTML(newsItem.title);
var worth = newsItem.isLocked ? "subscription" : "normal";
var retValue = {
sourceName: newsItem.source,
partnerCode: null,
contentId: newsItem.destination || newsItem.articleUrl,
type: type,
date: date,
uri: null,
slug: slug,
isSummary: true,
worth: worth, // TODO: WSJ and NYT should be premium if on Bing Daily, possibly others
isAd: false,
adCampaign: null,
};
return retValue;
},
_getAdsTelemetryInfo: function _getAdsTelemetryInfo(item) {
var retValue = {
isAd: true,
adCampaign: this._adUnitId,
date: new Date(1, 1, 1, 0, 0, 0, 0),
};
return retValue;
},
_convertTypeToTelemetryType: function _convertTypeToTelemetryType(type) {
var telemetryType;
if (this.isVideoClass(type)) {
telemetryType = "video";
}
else if (this.isSlideshowClass(type)) {
telemetryType = "slideshow";
}
else if (this.isArticleClass(type)) {
telemetryType = "article";
}
else {
telemetryType = "app-" + type;
}
return telemetryType;
},
_convertUtcTime: function _convertUtcTime(utctime) {
var dateValue = 0;
if (typeof (utctime) === "number") {
var epoch = Date.UTC(1601, 0, 1, 0, 0, 0);
dateValue = utctime / 10000 + epoch;
}
return new Date(dateValue);
},
_showNoResults: function _showNoResults() {
this._hideProgress();
this._hideError();
if (!this._noResultsElt) {
var element = document.createElement("div");
element.className = "noEntityResultsFoundMessage";
var row = document.createElement("div");
row.className = "row";
var image = document.createElement("div");
image.className = "image win-command";
var ring = document.createElement("div");
ring.className = "ring win-commandring win-commandicon";
var icon = document.createElement("div");
icon.className = "icon win-commandimage";
var message = document.createElement("div");
message.className = "message";
message.innerText = this._noResultsMessage;
ring.appendChild(icon);
image.appendChild(ring);
row.appendChild(image);
row.appendChild(message);
element.appendChild(row);
this._noResultsElt = element;
this._domElement.appendChild(element);
}
WinJS.Utilities.removeClass(this._noResultsElt, "platformHide");
},
_hideNoResults: function _hideNoResults() {
if (this._noResultsElt) {
WinJS.Utilities.addClass(this._noResultsElt, "platformHide");
}
},
_renderError: function _renderError(err) {
// show error message
this._showError(err);
// dispatch error event
this.dispatchEvent(ERROR, err);
// complete render promise
msWriteProfilerMark("CommonControls:EntityCluster:render:" + this._uniqueID + ":e");
// Marker for test automation
msWriteProfilerMark("CommonControls:EntityCluster:RenderComplete");
},
_showError: function _showError(err) {
this._createErrorModule(err);
this._hideProgress();
this._hideNoResults();
WinJS.Utilities.removeClass(this._errorElt, "platformHide");
},
_hideError: function _hideError() {
if (this._errorElt) {
WinJS.Utilities.addClass(this._errorElt, "platformHide");
}
},
_showProgress: function _showProgress() {
if (!this._progressElt || !this._progressElt.parentElement) {
this._createProgressModule();
}
this._hideNoResults();
this._hideError();
WinJS.Utilities.removeClass(this._progressElt, "platformHide");
},
_hideProgress: function _hideProgress() {
if (this._progressElt) {
WinJS.Utilities.addClass(this._progressElt, "platformHide");
}
},
_createErrorModule: function _createErrorModule(error) {
var that = this;
var data;
var retryFunction = WinJS.Utilities.markSupportedForProcessing(function (evt) {
if (that.retryHandler && that.mode === News.ClusterMode.static) {
that.dispatchEvent("retry", evt);
} else {
that.render();
}
});
var isFREOfflineError = error && error.message && error.message.indexOf("FREOffline") === 0;
if (!PlatformJS.Utilities.hasInternetConnection() || isFREOfflineError) {
data = CommonJS.Error.getErrorModuleItem(retryFunction, "450px", "100%", "/platform/offline_noContent", "platformOfflineErrorModule");
} else {
data = CommonJS.Error.getErrorModuleItem(retryFunction, "450px", "100%");
}
var moduleInfo = data.moduleInfo;
var domElement = this._domElement;
var errorElt = document.createElement("div");
errorElt.className = "platformHide";
errorElt.style.width = moduleInfo.width;
errorElt.style.height = moduleInfo.height;
domElement.innerHTML = "";
domElement.appendChild(errorElt);
CommonJS.loadModule(moduleInfo, data, errorElt);
this._errorElt = errorElt;
},
_createProgressModule: function _createProgressModule() {
var data = CommonJS.Error.getProgressModuleItem();
var moduleInfo = data.moduleInfo;
var domElement = this._domElement;
var progressElt = document.createElement("div");
progressElt.className = "platformHide";
progressElt.style.width = moduleInfo.width;
progressElt.style.height = moduleInfo.height;
domElement.appendChild(progressElt);
CommonJS.loadModule(moduleInfo, data, progressElt);
this._progressElt = progressElt;
},
_queryFromRenderedData: function _queryFromRenderedData(renderedData) {
/// <summary>Returns an object that conforms to the news provider return type from the given renderedData object.</summary>
return {
newsItems: renderedData
};
},
_processData: function _processData(queryResult) {
var data = null;
// if response is marshalled from managed, convert
if (queryResult && !queryResult.native) {
data = PlatformJS.Utilities.convertManageToJSON(queryResult);
} else {
data = queryResult;
}
return data;
},
_queryProviderAsync: function _queryProviderAsync() {
var that = this;
this._showProgress();
// start invalidation
var providerConfiguration = PlatformJS.Collections.createStringDictionary();
providerConfiguration.insert("queryServiceId", this.queryServiceId);
return new WinJS.Promise(function (complete, error, progress) {
that.provider.initializeAsync(providerConfiguration)
.then(function (initResult) {
if (!initResult) {
throw "NewsCluster initialization error.";
}
var query = PlatformJS.Collections.createStringDictionary();
for (var key in that.newsQuery) {
query.insert(key, that.newsQuery[key] + "");
}
return that.provider.queryAsync(query);
})
.then(
function (queryResult) {
var data = that._processData(queryResult);
complete(data);
},
function (e) {
error(e);
},
function (queryResult) {
var data = that._processData(queryResult);
progress(data);
});
});
},
_debugAppendLayoutInfo: function _debugAppendLayoutInfo(layout) {
if (this._debugTemplateSelectorData) {
var portrait = layout.portrait;
var landscape = layout.landscape;
if (portrait && portrait.debugData) {
var portraitDebug = portrait.debugData;
for (var i = 0, leni = portraitDebug.length; i < leni; i++) {
this._debugTemplateSelectorData.portrait[i] = this._debugTemplateSelectorData.portrait[i].concat(portraitDebug[i]);
}
}
if (landscape && landscape.debugData) {
var landscapeDebug = landscape.debugData;
for (var j = 0, lenj = landscapeDebug.length; j < lenj; j++) {
this._debugTemplateSelectorData.landscape[j] = this._debugTemplateSelectorData.landscape[j].concat(landscapeDebug[j]);
}
}
}
},
_writeToDebugData: function _writeToDebugData(text, index, isLandscape, isPortrait) {
if (this._debugTemplateSelectorData) {
if (isLandscape) {
this._debugTemplateSelectorData.landscape[index].push(text);
}
if (isPortrait) {
this._debugTemplateSelectorData.portrait[index].push(text);
}
}
},
_handleDebugOnClick: function _handleDebugOnClick(index, element) {
var flyout = document.getElementById("entityClusterDebugFlyout");
var labelElt, flyoutControl;
if (!flyout) {
flyout = CommonJS.FlyoutManager.getFlyoutElement();
flyout.id = "entityClusterDebugFlyout";
labelElt = document.createElement("div");
labelElt.id = "entityClusterDebugFlyoutLabel";
flyout.appendChild(labelElt);
flyoutControl = new WinJS.UI.Flyout(flyout, {
alignment: "left"
});
}
else {
labelElt = document.getElementById("entityClusterDebugFlyoutLabel");
flyoutControl = flyout.winControl;
}
if (flyout && flyoutControl) {
var value = Windows.UI.ViewManagement.ApplicationView.value;
var currentOrientationDebugData;
if (value === Windows.UI.ViewManagement.ApplicationViewState.fullScreenPortrait) {
currentOrientationDebugData = this._debugTemplateSelectorData.portrait;
}
else {
currentOrientationDebugData = this._debugTemplateSelectorData.landscape;
}
labelElt.innerText = currentOrientationDebugData[index].join("");
flyoutControl.show(element, "right");
}
},
_addSupplementalConfigs: function _addSupplementalConfigs() {
var supplemental = CommonJS.News.SupplementalConfig;
if (supplemental && supplemental.Config && !this._supplementalConfigCacheKey) {
this._supplementalConfigCacheKey = ["SupplementalConfig_", supplemental.Version].join("");
var supplementalRulesList = supplemental.Config.RulesList;
var supplementalTemplateClassMap = supplemental.Config.TemplateClassMap;
var supplementalTemplateDefinitions = supplemental.Config.TemplateDefinitions;
if (supplementalRulesList) {
this._rulesList = this._rulesList.concat(supplementalRulesList);
}
if (supplementalTemplateClassMap) {
this._templateClassMap = this._mergeObject(this._templateClassMap, supplementalTemplateClassMap);
}
if (supplementalTemplateDefinitions) {
this._templateDefinitions = this._mergeObject(this._templateDefinitions, supplementalTemplateDefinitions);
}
}
},
_mergeObject: function _mergeObject(obj1, obj2) {
var retVal = {};
this._copyObject(retVal, obj1);
this._copyObject(retVal, obj2);
return retVal;
},
_copyObject: function _copyObject(retVal, obj) {
for (var name in obj) {
retVal[name] = obj[name];
}
},
getLastKnownGoodState: function getLastKnownGoodState() {
var key = this._generateLayoutCacheKey("stateTest");
var itemWrappers = this._constructGoodStateItemWrappers();
var layout = this._layout;
if (layout && itemWrappers.length > 0) {
return {
itemWrappers: itemWrappers,
layout: layout,
videoList: this._videoList,
key: key,
adsOptions: {
subclusterAdFrequency: this.subclusterAdFrequency,
adsList: this._adsList,
adUnitId: this._adUnitId,
}
};
}
else {
return null;
}
},
isLastKnownGoodStateValid: function isLastKnownGoodStateValid(state) {
var result = false;
if (state) {
this._addSupplementalConfigs();
var newKey = this._generateLayoutCacheKey("stateTest");
if (newKey === state.key && state.layout && state.itemWrappers) {
result = true;
}
}
return result;
},
renderLastKnownGoodState: function renderLastKnownGoodState(state) {
if (state) {
this._addSupplementalConfigs();
var newKey = this._generateLayoutCacheKey("stateTest");
var itemWrappers = this._itemWrappers = state.itemWrappers;
var layout = this._layout = state.layout;
var adsOptions = state.adsOptions;
var videoList = state.videoList;
var dimensions = this._getDimensions();
this._currentHeight = dimensions.height;
this._currentWidth = dimensions.width;
if (newKey === state.key && itemWrappers && layout) {
this.subclusterAdFrequency = adsOptions.subclusterAdFrequency;
this._adsList = adsOptions.adsList;
this._adUnitId = adsOptions.adUnitId;
this._createDebugData(itemWrappers);
this._computeItemsToRender(layout, itemWrappers, videoList);
this._renderClasses(layout);
}
}
},
_constructGoodStateItemWrappers: function _constructGoodStateItemWrappers() {
var itemWrappers = this._itemWrappers;
var retVal = [];
if (itemWrappers) {
for (var i = 0, leni = itemWrappers.length; i < leni; i++) {
var itemWrapper = itemWrappers[i];
var item = itemWrapper.item;
var newItem = {
altText: item.altText,
articleid: item.articleid,
contentid: item.contentid,
destination: item.destination,
editorial: item.editorial,
imageAttribution: item.imageAttribution,
published: item.published,
publishTime: item.publishTime,
snippet: item.snippet,
source: item.source,
sourceImageUrl: item.sourceImageUrl,
telemetry: item.telemetry,
templateClass: item.templateClass,
thumbnail: item.thumbnail,
title: item.title,
type: item.type,
updated: item.updated,
updatedTime: item.updatedTime,
};
var newItemWrapper = {
index: itemWrapper.index,
templateClass: itemWrapper.templateClass,
item: newItem,
};
retVal.push(newItemWrapper);
}
}
return retVal;
},
_resizeLastKnownGoodState: function _resizeLastKnownGoodState() {
var that = this;
var videoList = this.videoList;
var itemWrappers = this._itemWrappers;
// we don't have cache info in good state mode
return that._layoutGeneration(itemWrappers, null, null)
.then(function (layout) {
msWriteProfilerMark("CommonControls:EntityCluster:generateLayoutNormal:e");
that._computeItemsToRender(layout, itemWrappers, videoList);
return that._renderClasses(layout);
});
},
setOptionsLastKnownGoodState: function setOptionsLastKnownGoodState(options) {
var unitWidth = this._unitWidth = options.unitWidth ? options.unitWidth : 31;
this._unitHeight = options.unitHeight ? options.unitHeight : 15;
this._noResultsMessage = options.noResultsMessage ? options.noResultsMessage : PlatformJS.Services.resourceLoader.getString("/platform/invalidEntry");
// set maxWidth on the view if maxColumnCount is provided in options
var maxColumnCount = options.maxColumnCount;
if (maxColumnCount) {
this._maxWidth = maxColumnCount * unitWidth - 1;
}
if (options.marginDistance && options.marginWidth) {
this._marginDistance = options.marginDistance * unitWidth - 1;
this._marginWidth = options.marginWidth;
if (maxColumnCount) {
var marginOccurances = Math.floor(options.maxColumnCount / options.marginDistance);
this._maxWidth += (marginOccurances * this._marginWidth);
}
}
this._adUnitId = options.adUnitId;
this._adsList = options.adsList;
this._adsWrapperControlList = [];
this._progressElt = null;
this._errorElt = null;
this._templateClassMap = options.configObject.TemplateClassMap;
this._templateDefinitions = options.configObject.TemplateDefinitions;
this._rulesList = options.configObject.RulesList || [];
this._leadingItemIsLarge = false;
this._instrumentationEntryPoint = options.instrumentationEntryPoint;
// set options
WinJS.UI.setOptions(this, options);
},
}, {
TemplateSelectorDebuggingEnabled: false,
itemClickHandler: function itemClickHandler(evt) {
// launch url in browser
var item = evt.item;
var url = item.articleUrl;
if (url) {
var uri = new Windows.Foundation.Uri(url);
Windows.System.Launcher.launchUriAsync(uri).then();
}
}
// end static type definition
}), WinJS.UI.DOMEventMixin)
});
})(WinJS);
/* File: LayoutGenerator.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function (WinJS) {
"use strict";
var DEBUG = PlatformJS.isDebug;
var News = WinJS.Namespace.define("CommonJS.News",
{
LayoutGenerator: WinJS.Class.define(function (options) {
// ctor
this._options = options;
this._layout = {
itemList: [],
adList: [],
orderedRenderList: [],
};
var rowCount = this._layout.rowCount = this._rowCount = Math.floor(options.screenHeight / CommonJS.News.LayoutGenerator.RowHeight);
this._minColumnWidth = options.unitWidth;
this._minRowHeight = options.unitHeight;
this._setupRules();
if (rowCount > 0) {
this._cells = [];
for (var i = 0; i < rowCount; i++) {
var cell = {
row: i,
column: 0,
width: 0,
height: 0,
isAvailable: true,
isMargin: false
};
this._cells.push([cell]);
}
this._currentExpandCell = null;
this._nextExpandCell = this._cells[0][0];
this._stopArranging = false;
this._enableSeeMore = options.enableSeeMore;
}
},
{
// declare null aliases for auto-complete to work
_options: null,
_rowCount: null,
_layout: null,
_minColumnWidth: null,
_minRowHeight: null,
_cells: null,
_currentExpandCell: null,
_nextExpandCell: null,
_stopArranging: null,
_rulesList: null,
_ruleExecutorList: null,
_columnCount: 0,
_marginCount: 0,
_subclusterAdCount: 0,
_lastMargin: 0,
_enableSeeMore: null,
generateLayout: function generateLayout(templateData) {
if (this._rowCount > 0) {
if (DEBUG) {
this._setupDebugData(templateData);
}
for (var i = 0, leni = templateData.length; i < leni; i++) {
if (this._stopArranging || !templateData[i]) {
this._layout.itemList.push(null);
if (DEBUG) {
this._writeDebugData("This item was not arranged in the layout.\n", i);
}
}
else {
this._arrangeItem(templateData[i], i);
}
}
var enableSeeMore = this._enableSeeMore;
if (enableSeeMore) {
var minRowHeight = this._minRowHeight;
var minColumnWidth = this._minColumnWidth;
var rowSpan = minRowHeight - 1;
var columnSpan = minColumnWidth - 1;
var row = Math.floor((this._rowCount + 1) / minRowHeight) * minRowHeight - rowSpan;
var column = Math.floor((this._columnCount + 2) / minColumnWidth) * minColumnWidth - columnSpan;
if (column + columnSpan === this._options.maxWidth + 1) {
this._layout.seeMore = {
row: row,
column: column,
rowSpan: rowSpan,
columnSpan: columnSpan,
};
}
}
}
this._layout.columnCount = this._columnCount;
this._enrichItemLayoutInfo();
return this._layout;
},
_writeDebugData: function _writeDebugData(text, index) {
this._layout.debugData[index].push(text);
},
_setupDebugData: function _setupDebugData(templateData) {
this._layout.debugData = [];
for (var i = 0, leni = templateData.length; i < leni; i++) {
this._layout.debugData.push([""]);
}
},
_enrichItemLayoutInfo: function _enrichItemLayoutInfo() {
for (var i = 0, leni = this._layout.itemList.length; i < leni; i++) {
var item = this._layout.itemList[i];
if (item) {
this._checkBottomRow(item);
this._checkTopRow(item);
this._checkLeftColumn(item);
this._checkRightColumn(item);
}
}
},
_checkTopRow: function _checkTopRow(item) {
if (item.row === 1) {
item.clusterLocation += " topRow";
}
},
_checkBottomRow: function _checkBottomRow(item) {
var cells = this._cells;
if (cells.length > 0) {
var newY = item.row + item.rowSpan - 1;
if (newY >= cells.length) {
item.clusterLocation += " bottomRow";
return;
}
var c = cells[newY][item.column];
if (c && c.isAvailable) {
item.clusterLocation += " bottomRow";
return;
}
}
},
_checkLeftColumn: function _checkLeftColumn(item) {
if (item.column === 1) {
item.clusterLocation += " leftColumn";
return;
}
var cells = this._cells;
if (cells.length > 0) {
var newX = item.column - 3;
var c = cells[item.row][newX];
if (c && (c.isAvailable || c.isMargin)) {
item.clusterLocation += " leftColumn";
return;
}
}
},
_checkRightColumn: function _checkRightColumn(item) {
var cells = this._cells;
if (cells.length > 0) {
var newX = item.column + item.columnSpan;
if (newX >= cells[0].length) {
item.clusterLocation += " rightColumn";
return;
}
var c = cells[item.row][newX];
if (c && (c.isAvailable || c.isMargin)) {
item.clusterLocation += " rightColumn";
return;
}
}
},
_arrangeItem: function _arrangeItem(templateList, itemIndex) {
if (this._options.marginDistance && this._options.marginWidth) {
var c = this._getNextAvailableCellRange();
if (this._nextExpandCell.column - this._lastMargin >= this._options.marginDistance && this._options.marginWidth > 2 && c.row === 0) {
this._marginCount++;
this._tryCreateSubclusterAd();
var margin = {
row: 0,
column: this._nextExpandCell.column,
width: 0,
height: 0,
isAvailable: true,
isMargin: false
};
var templateInfo = {
classID: "Margin",
height: this._rowCount,
width: this._options.marginWidth - 2
};
this._updateCells(margin, templateInfo, true);
this._lastMargin = this._nextExpandCell.column;
this._clearRuleData({ height: 0, width: this._options.marginWidth }, margin);
}
}
var arranged = this._tryArrangeItem(templateList, itemIndex);
if (!arranged && !this._stopArranging) {
var oldExpandCell = this._currentExpandCell;
this._currentExpandCell = this._nextExpandCell;
var addedSpace = false;
if (this._options.marginDistance && this._options.marginWidth && this._currentExpandCell && this._currentExpandCell.row === 0) {
if (this._currentExpandCell.column - this._lastMargin >= this._options.marginDistance && this._options.marginWidth > 2) {
this._marginCount++;
var addedAd = this._tryCreateSubclusterAd();
var column = addedAd ? this._nextExpandCell.column : this._currentExpandCell.column;
var margin2 = {
row: 0,
column: column,
width: 0,
height: 0,
isAvailable: true,
isMargin: false
};
var templateInfo2 = {
classID: "Margin",
height: this._rowCount,
width: this._options.marginWidth - 2
};
this._updateCells(margin2, templateInfo2, true);
this._lastMargin = this._currentExpandCell.column;
this._clearRuleData({ height: 0, width: this._options.marginWidth }, margin2);
addedSpace = true;
}
}
arranged = this._tryArrangeItem(templateList, itemIndex);
if (!arranged && !addedSpace) {
this._currentExpandCell = oldExpandCell;
}
}
if (!arranged) {
this._layout.itemList.push(null);
if (DEBUG) {
this._writeDebugData("This item was not arranged in the layout.\n", itemIndex);
}
}
},
_tryArrangeItem: function _tryArrangeItem(templateList, itemIndex) {
var arranged = false;
var cellRange = this._getNextAvailableCellRange();
if (cellRange && !this._stopArranging) {
var availableWidth = (cellRange.column === 0 ? cellRange.width : cellRange.width - CommonJS.News.LayoutGenerator.ColumnMargin);
var availableHeight = (cellRange.row === 0 ? cellRange.height : cellRange.height - CommonJS.News.LayoutGenerator.RowMargin);
var satisfiedList = [];
for (var i = 0, leni = templateList.length; i < leni; i++) {
var current = templateList[i];
if (current && current.height <= availableHeight && current.width <= availableWidth) {
var satisfiedRules = true;
for (var j = 0, lenj = this._ruleExecutorList.length; j < lenj; j++) {
var executor = this._ruleExecutorList[j];
satisfiedRules = satisfiedRules && this._adhereToRules(executor, current, cellRange, itemIndex);
if (!satisfiedRules) {
break;
}
}
if (satisfiedRules) {
var weight = current.weight || 1;
for (var k = 0; k < weight; k++) {
satisfiedList.push(current);
}
}
}
}
var chosen;
var itemLayout;
//HACK: Slideshow + Video leading item is the largest template
if (itemIndex === 0 && this._options.leadingItemIsLarge) {
chosen = this._tryArrangeLeadingItemIsLarge(satisfiedList);
if (chosen) {
arranged = true;
this._updateCells(cellRange, chosen, false);
itemLayout = {
classID: chosen.classID,
row: cellRange.row + 1 + (cellRange.row === 0 ? 0 : CommonJS.News.LayoutGenerator.RowMargin),
column: cellRange.column + 1 + (cellRange.column === 0 ? 0 : CommonJS.News.LayoutGenerator.ColumnMargin),
rowSpan: chosen.height,
columnSpan: chosen.width,
thumbnailModifier: chosen.thumbnailModifier,
clusterLocation: ""
};
if (DEBUG) {
this._writeDebugData("The template that was chosen was: " + chosen.classID + ", because we chose the largest possible template for the first slideshow/video.\n", itemIndex);
}
this._layout.itemList.push(itemLayout);
this._updateOrderedRenderList(itemIndex, false);
this._updateRuleData(chosen, cellRange);
if (this._columnCount < chosen.width + cellRange.column) {
this._columnCount = chosen.width + cellRange.column;
}
}
}
else {
while (satisfiedList.length > 0) {
var index = Math.floor(Math.random() * satisfiedList.length);
chosen = satisfiedList[index];
if (chosen.lastResort) {
if (satisfiedList.length === 1) {
arranged = true;
}
}
else {
arranged = true;
}
if (arranged) {
this._updateCells(cellRange, chosen, false);
itemLayout = {
classID: chosen.classID,
row: cellRange.row + 1 + (cellRange.row === 0 ? 0 : CommonJS.News.LayoutGenerator.RowMargin),
column: cellRange.column + 1 + (cellRange.column === 0 ? 0 : CommonJS.News.LayoutGenerator.ColumnMargin),
rowSpan: chosen.height,
columnSpan: chosen.width,
thumbnailModifier: chosen.thumbnailModifier,
clusterLocation: ""
};
if (DEBUG) {
this._writeDebugData("The template that was chosen was: " + chosen.classID + ".\n", itemIndex);
}
this._layout.itemList.push(itemLayout);
this._updateOrderedRenderList(itemIndex, false);
this._updateRuleData(chosen, cellRange);
if (this._columnCount < chosen.width + cellRange.column) {
this._columnCount = chosen.width + cellRange.column;
}
break;
}
else {
satisfiedList.splice(index, 1);
}
}
}
}
return arranged;
},
_tryArrangeLeadingItemIsLarge: function _tryArrangeLeadingItemIsLarge(satisfiedList) {
var chosen;
var currentLargestArea = 0;
for (var i = 0, leni = satisfiedList.length; i < leni; i++) {
var item = satisfiedList[i];
var area = item.width * item.height;
if (area > currentLargestArea) {
chosen = item;
currentLargestArea = area;
}
}
return chosen;
},
_updateCells: function _updateCells(cellRange, templateInfo, isMargin) {
var row = cellRange.row;
var column = cellRange.column;
var cell = this._cells[row][column];
var width = templateInfo.width;
var height = templateInfo.height;
var leni, lenj, i, j;
if (this._cellsAreEqual(cell, this._nextExpandCell)) {
this._currentExpandCell = this._nextExpandCell;
leni = this._rowCount;
lenj = this._currentExpandCell.column + width + 1 + (column === 0 ? 0 : CommonJS.News.LayoutGenerator.ColumnMargin);
var startColumn = this._currentExpandCell.column + 1;
for (i = 0; i < leni; i++) {
for (j = startColumn; j < lenj; j++) {
var newCell = {
row: i,
column: j,
width: 0,
height: 0,
isAvailable: true,
isMargin: false
};
this._cells[i].push(newCell);
}
}
this._nextExpandCell = this._cells[0][this._currentExpandCell.column + width + (column === 0 ? 0 : CommonJS.News.LayoutGenerator.ColumnMargin)];
}
leni = (row === 0 ? row + height : row + height + CommonJS.News.LayoutGenerator.RowMargin);
lenj = (column === 0 ? column + width : column + width + CommonJS.News.LayoutGenerator.ColumnMargin);
for (i = row; i < leni; i++) {
for (j = column; j < lenj; j++) {
this._cells[i][j].isAvailable = false;
this._cells[i][j].width = width;
this._cells[i][j].height = height;
this._cells[i][j].isMargin = isMargin;
}
}
},
_getNextAvailableCellRange: function _getNextAvailableCellRange() {
var nextCell = null;
var cellRange = null;
var i, j;
if (this._currentExpandCell) {
for (i = 0; i < this._rowCount; i++) {
for (j = this._currentExpandCell.column; j < this._nextExpandCell.column; j++) {
var cell = this._cells[i][j];
if (cell.isAvailable) {
nextCell = cell;
break;
}
}
if (nextCell) {
break;
}
}
}
if (nextCell) {
var width = 0;
var cellRow = this._cells[nextCell.row];
var previousRow = this._cells[nextCell.row - 1];
for (i = nextCell.column; i < this._nextExpandCell.column; i++) {
if (cellRow[i].isAvailable && !previousRow[i].isAvailable) {
width++;
}
else {
break;
}
}
var height = this._rowCount - nextCell.row;
cellRange = {
row: nextCell.row,
column: nextCell.column,
width: width,
height: height,
isAvailable: true,
isMargin: false
};
}
else if (this._nextExpandCell) {
var availableWidth = this._options.maxWidth ? this._options.maxWidth - this._nextExpandCell.column : Number.MAX_VALUE;
if (availableWidth <= 0) {
this._stopArranging = true;
}
cellRange = {
row: 0,
column: this._nextExpandCell.column,
width: availableWidth,
height: this._rowCount,
isAvailable: true,
isMargin: false
};
}
return cellRange;
},
_tryCreateSubclusterAd: function _tryCreateSubclusterAd() {
var options = this._options;
if (options.subclusterAdFrequency) {
if ((this._marginCount % options.subclusterAdFrequency) === 0) {
if (options.subclusterAdMax) {
if (this.subclusterAdCount < options.subclusterAdMax) {
this._createSubclusterAd();
return true;
}
}
else {
this._createSubclusterAd();
return true;
}
}
}
return false;
},
_createSubclusterAd: function _createSubclusterAd() {
if (this._options.maxWidth) {
this._options.maxWidth += 31;
}
var ad = {
row: 0,
column: this._nextExpandCell.column,
width: 0,
height: 0,
isAvailable: true,
isMargin: false
};
var itemLayout = {
classID: "SubclusterAd",
row: 1,
column: this._nextExpandCell.column + 1 + (this._nextExpandCell.column === 0 ? 0 : CommonJS.News.LayoutGenerator.ColumnMargin),
rowSpan: this._rowCount,
columnSpan: 30,
thumbnailModifier: "",
clusterLocation: ""
};
var templateInfo = {
classID: "Ad",
height: this._rowCount,
width: 30
};
this._layout.adList.push(itemLayout);
this._updateOrderedRenderList(null, true);
this._updateCells(ad, templateInfo, true);
if (this._columnCount < 30 + ad.column) {
this._columnCount = 30 + ad.column;
}
this._subclusterAdCount++;
},
_updateOrderedRenderList: function _updateOrderedRenderList(index, isAd) {
var orderedRenderList = this._layout.orderedRenderList;
orderedRenderList.push({
realIndex: index,
isAd: isAd
});
},
_setupRules: function _setupRules() {
var rules = this._rulesList = this._options.rulesList;
this._ruleExecutorList = [];
for (var i = 0, leni = rules.length; i < leni; i++) {
var rule = rules[i];
var executor = this._createRuleExecutor(rule);
if (executor) {
this._ruleExecutorList.push(executor);
}
}
},
_clearRuleData: function _clearRuleData(templateInfo, cellRange) {
for (var i = 0, leni = this._ruleExecutorList.length; i < leni; i++) {
var executor = this._ruleExecutorList[i];
this._clearIndividualRuleData(executor, templateInfo, cellRange);
}
},
_updateRuleData: function _updateRuleData(templateInfo, cellRange) {
for (var i = 0, leni = this._ruleExecutorList.length; i < leni; i++) {
var executor = this._ruleExecutorList[i];
this._updateIndividualRuleData(executor, templateInfo, cellRange);
}
},
_createRuleExecutor: function _createRuleExecutor(rule) {
if (!rule) {
return null;
}
switch (rule.type) {
case "Frequency":
return {
type: rule.type,
rule: rule,
totalTemplates: 0,
realZero: 0
};
case "Position":
return {
type: rule.type,
rule: rule,
realZero: 0
};
default:
return null;
}
},
_clearIndividualRuleData: function _clearIndividualRuleData(executor, templateInfo, cellRange) {
switch (executor.type) {
case "Frequency":
executor.totalTemplates = 0;
executor.realZero = cellRange.column + templateInfo.width;
return;
case "Position":
executor.realZero = cellRange.column + templateInfo.width;
return;
default:
return;
}
},
_updateIndividualRuleData: function _updateIndividualRuleData(executor, templateInfo, cellRange) {
switch (executor.type) {
case "Frequency":
var columns = Math.floor((cellRange.column - executor.realZero + 1) / this._minColumnWidth);
var rule = executor.rule;
if (rule.columns > 0) {
if (cellRange.row === 0 && columns >= rule.columns) {
executor.totalTemplates = 0;
executor.realZero = cellRange.column;
}
}
for (var i = 0; i < rule.templateList.length; i++) {
var current = rule.templateList[i];
if (current === templateInfo.classID) {
executor.totalTemplates++;
}
}
return;
case "Position":
return;
default:
return;
}
},
_adhereToRules: function _adhereToRules(executor, templateInfo, cellRange, itemIndex) {
switch (executor.type) {
case "Frequency":
return this._adhereFrequencyRule(executor, templateInfo, cellRange, itemIndex);
case "Position":
return this._adherePositionRule(executor, templateInfo, cellRange, itemIndex);
default:
return true;
}
},
_adhereFrequencyRule: function _adhereFrequencyRule(executor, templateInfo, cellRange, itemIndex) {
var rule = executor.rule;
for (var i = 0, leni = rule.templateList.length; i < leni; i++) {
var current = rule.templateList[i];
if (current === templateInfo.classID) {
if (executor.totalTemplates + 1 <= rule.count) {
return true;
}
else {
if (DEBUG) {
this._writeDebugData("The template: " + templateInfo.classID + " was thrown out because it didn't satisfy a frequency rule.\n", itemIndex);
}
return false;
}
}
}
return true;
},
_adherePositionRule: function _adherePositionRule(executor, templateInfo, cellRange, itemIndex) {
var rule = executor.rule;
for (var i = 0, leni = rule.templateList.length; i < leni; i++) {
var current = rule.templateList[i];
if (current === "*" || current === templateInfo.classID) {
var column = Math.floor((cellRange.column - executor.realZero + 1) / this._minColumnWidth);
var columnEnd = column + (templateInfo.width + 1) / this._minColumnWidth - 1;
var row = Math.floor((cellRange.row + 1) / this._minRowHeight);
var rowEnd = row + (templateInfo.height + 1) / this._minRowHeight - 1;
if (rule.isRestrictedArea) {
if((rowEnd >= rule.minY && rowEnd <= rule.maxY && columnEnd >= rule.minX && columnEnd <= rule.maxX)){
if (DEBUG) {
this._writeDebugData("The template: " + templateInfo.classID + " was thrown out because it didn't satisfy a position area rule.\n", itemIndex);
}
return false;
}
}
else {
if ((row >= rule.minY && row <= rule.maxY && column >= rule.minX && column <= rule.maxX)){
if (DEBUG) {
this._writeDebugData("The template: " + templateInfo.classID + " was thrown out because it didn't satisfy a position rule.\n", itemIndex);
}
return false;
}
}
}
}
return true;
},
_cellsAreEqual: function _cellsAreEqual(x, y) {
if (!x || !y) {
return false;
}
return (x.row === y.row) && (x.column === y.column) && (x.width === y.width) && (x.height === y.height) && (x.isAvailable === y.isAvailable);
}
}, {
RowMargin: 1,
ColumnMargin: 1,
RowHeight: 10,
ColumnWidth: 10
// end static type definition
})
});
})(WinJS);
/* File: OrientationLayoutGenerator.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function (WinJS) {
"use strict";
var News = WinJS.Namespace.define("CommonJS.News",
{
OrientationLayoutGenerator: {
generateLayout: function generateLayout(templateData, currentOptions, otherScreenHeight, otherRulesList, currentIsLandscape) {
var otherOptions = currentOptions;
var currentOrientation = new CommonJS.News.LayoutGenerator(currentOptions);
var currentJson = currentOrientation.generateLayout(templateData);
otherOptions.screenHeight = otherScreenHeight;
otherOptions.rulesList = otherRulesList;
if (currentJson.adList.length > 0) {
otherOptions.subclusterAdMax = currentJson.adList.length;
}
var otherOrientation = new CommonJS.News.LayoutGenerator(otherOptions);
var otherJson = otherOrientation.generateLayout(templateData);
var layout = {};
if (currentIsLandscape) {
layout.landscape = currentJson;
layout.portrait = otherJson;
}
else {
layout.portrait = currentJson;
layout.landscape = otherJson;
}
var string = JSON.stringify(layout);
return WinJS.Promise.wrap(string);
}
}
});
})(WinJS);
